@@ -1,2 +1,4 @@
 #! /usr/bin/env bash
+$EXTRACTRC *.ui >> rc.cpp
 $XGETTEXT *.cpp -o $podir/kcmbackground.pot
+rm -f rc.cpp
