@@ -22,6 +22,7 @@
    Boston, MA 02110-1301, USA.
 */
 
+#include <qglobal.h>
 #ifdef Q_WS_X11
 #include <config-X11.h>
 #endif
@@ -191,11 +192,6 @@ Klipper::Klipper(QObject *parent, const KSharedConfigPtr &config)
     // if it's too old, drop it and just use load history
     readProperties(m_config.data());
 
-    KAction* a = m_collection->addAction("show_klipper_popup");
-    a->setText(i18n("Show Klipper Popup-Menu"));
-    a->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_V));
-    connect(a, SIGNAL(triggered()), SLOT(slotPopupMenu()));
-
     m_repeatAction = m_collection->addAction("repeat_action");
     m_repeatAction->setText(i18n("Manually Invoke Action on Current Clipboard"));
     qobject_cast<KAction*>(m_repeatAction)->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_R));
@@ -210,11 +206,11 @@ Klipper::Klipper(QObject *parent, const KSharedConfigPtr &config)
 
     // Cycle through history
     m_cycleNextAction = m_collection->addAction("cycleNextAction");
-    m_cycleNextAction->setText(i18n("Next history item"));
+    m_cycleNextAction->setText(i18n("Next History Item"));
     m_cycleNextAction->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_Down), KAction::DefaultShortcut);
     connect(m_cycleNextAction, SIGNAL(triggered(bool)), SLOT(slotCycleNext()));
     m_cyclePrevAction = m_collection->addAction("cyclePrevAction");
-    m_cyclePrevAction->setText(i18n("Previous history item"));
+    m_cyclePrevAction->setText(i18n("Previous History Item"));
     m_cyclePrevAction->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_Up), KAction::DefaultShortcut);
     connect(m_cyclePrevAction, SIGNAL(triggered(bool)), SLOT(slotCyclePrev()));
 
@@ -308,7 +304,6 @@ void Klipper::loadSettings()
         saveHistory(true);
     }
 
-    m_bPopupAtMouse = KlipperSettings::popupAtMousePosition();
     m_bKeepContents = KlipperSettings::keepClipboardContents();
     m_bReplayActionInHistory = KlipperSettings::replayActionInHistory();
     m_bNoNullClipboard = KlipperSettings::preventEmptyClipboard();
@@ -356,27 +351,14 @@ void Klipper::showPopupMenu( QMenu *menu )
     Q_ASSERT( menu != 0L );
 
     QSize size = menu->sizeHint(); // geometry is not valid until it's shown
-    if (m_bPopupAtMouse) {
-        QPoint g = QCursor::pos();
-        if ( size.height() < g.y() )
-            menu->popup(QPoint( g.x(), g.y() - size.height()));
-        else
-            menu->popup(QPoint(g.x(), g.y()));
-    } else {
-        if( KSystemTrayIcon* tray = dynamic_cast< KSystemTrayIcon* >( parent())) {
-            QRect g = tray->geometry();
-            QRect screen = KGlobalSettings::desktopGeometry(g.center());
+    QPoint pos = QCursor::pos();
+    // ### We can't know where the systray icon is (since it can be hidden or shown 
+    //     in several places), so the cursor position is the only option.
 
-            if ( g.x()-screen.x() > screen.width()/2 &&
-                 g.y()-screen.y() + size.height() > screen.height() )
-                menu->popup(QPoint( g.x(), g.y() - size.height()));
-            else
-                menu->popup(QPoint( g.x() + g.width(), g.y() + g.height()));
-        } else
-            abort();
+    if ( size.height() < pos.y() )
+        pos.ry() -= size.height();
 
-        //      menu->exec(mapToGlobal(QPoint( width()/2, height()/2 )));
-    }
+    menu->popup(pos);
 }
 
 bool Klipper::loadHistory() {
@@ -794,7 +776,7 @@ void Klipper::checkClipData( bool selectionMode )
 #endif
     const QMimeData* data = m_clip->mimeData( selectionMode ? QClipboard::Selection : QClipboard::Clipboard );
     if ( !data ) {
-        kWarning("No data in clipboard. This not not supposed to happen." );
+        kWarning() << "No data in clipboard. This not not supposed to happen.";
         return;
     }
     // TODO: Rewrite to Qt4 !!!
@@ -1187,14 +1169,11 @@ QString Klipper::cycleText() const
 }
 
 // http://bugreports.qt.nokia.com/browse/QTBUG-8157
-#if KDE_IS_VERSION( 4, 4, 85 )
+#if KDE_IS_VERSION( 4, 5, 85 )
 #warning Check if this is still needed, hopefully not.
 #endif
 KlipperEmptyDetector::KlipperEmptyDetector()
 : m_xfixes_event_base( -1 )
-#ifdef Q_WS_X11
-, m_xa_clipboard( None )
-#endif
 {
 #ifdef HAVE_XFIXES
     m_xa_clipboard = XInternAtom( QX11Info::display(), "CLIPBOARD", False );
