@@ -25,18 +25,18 @@
 #include <math.h>
 
 #include <QAction>
-#include <QTimeLine>
-#include <QDebug>
-#include <QtGlobal>
-#include <QTimer>
 #include <QApplication>
-#include <QPainter>
-#include <QStyleOptionGraphicsItem>
+#include <QDebug>
 #include <QGraphicsItemAnimation>
 #include <QGraphicsLinearLayout>
 #include <QGraphicsScene>
 #include <QGraphicsSceneMouseEvent>
 #include <QGraphicsView>
+#include <QPainter>
+#include <QPropertyAnimation>
+#include <QStyleOptionGraphicsItem>
+#include <QTimeLine>
+#include <QTimer>
 
 #include <KDebug>
 #include <KIcon>
@@ -53,23 +53,25 @@ void shadowBlur(QImage &image, int radius, const QColor &color);
 
 int ResultItem::s_fontHeight = 0;
 
-ResultItem::ResultItem(const Plasma::QueryMatch &match, Plasma::RunnerManager *runnerManager, QGraphicsWidget *parent)
+ResultItem::ResultItem(const SharedResultData *sharedData, const Plasma::QueryMatch &match, Plasma::RunnerManager *runnerManager, QGraphicsWidget *parent)
     : QGraphicsWidget(parent),
       m_match(0),
       m_configButton(0),
       m_highlight(0),
       m_index(-1),
-      m_highlightTimerId(0),
-      m_mouseHovered(false),
       m_configWidget(0),
       m_actionsWidget(0),
       m_actionsLayout(0),
-      m_runnerManager(runnerManager)
+      m_runnerManager(runnerManager),
+      m_sharedData(sharedData),
+      m_mouseHovered(false)
 {
+    m_highlightCheckTimer.setInterval(0);
+    m_highlightCheckTimer.setSingleShot(true);
+    connect(&m_highlightCheckTimer, SIGNAL(timeout()), this, SLOT(checkHighlighting()));
     setFlag(QGraphicsItem::ItemIsFocusable);
     setFlag(QGraphicsItem::ItemIsSelectable);
-    // We will accept hover events later on;
-    setAcceptHoverEvents(false);
+    setAcceptHoverEvents(true);
     setFocusPolicy(Qt::TabFocus);
     setCacheMode(DeviceCoordinateCache);
     setZValue(0);
@@ -81,6 +83,11 @@ ResultItem::ResultItem(const Plasma::QueryMatch &match, Plasma::RunnerManager *r
         //kDebug() << "font height is: " << s_fontHeight;
     }
 
+    m_highlightAnim = new QPropertyAnimation(this, "highlightState", this);
+    m_highlightAnim->setStartValue(0);
+    m_highlightAnim->setEndValue(1);
+    m_highlightAnim->setDuration(150);
+    m_highlightAnim->setEasingCurve(QEasingCurve::OutCubic);
     setMatch(match);
 }
 
@@ -112,6 +119,11 @@ QGraphicsWidget* ResultItem::arrangeTabOrder(QGraphicsWidget* last)
     return currentWidget;
 }
 
+bool ResultItem::isValid() const
+{
+    return m_match.isValid() || m_match.type() == Plasma::QueryMatch::InformationalMatch;
+}
+
 void ResultItem::setMatch(const Plasma::QueryMatch &match)
 {
     m_match = match;
@@ -158,6 +170,11 @@ void ResultItem::setMatch(const Plasma::QueryMatch &match)
 
     setupActions();
     calculateSize();
+
+    if (!m_match.isValid() && isSelected() && scene()) {
+        scene()->clearSelection();
+    }
+
     update();
 }
 
@@ -205,10 +222,8 @@ bool ResultItem::eventFilter(QObject *obj, QEvent *event)
         } else if (event->type() == QEvent::FocusIn) {
             focusInEvent(static_cast<QFocusEvent*>(event));
             actionButton->setAutoRaise(false);
-            update();
         } else if (event->type() == QEvent::GraphicsSceneHoverLeave || event->type() == QEvent::FocusOut) {
             actionButton->setAutoRaise(true);
-            update();
         } else if (event->type() == QEvent::KeyPress) {
             QKeyEvent* keyEvent = static_cast<QKeyEvent*>(event);
             if (keyEvent->key() == Qt::Key_Enter || keyEvent->key() == Qt::Key_Return) {
@@ -320,6 +335,10 @@ void ResultItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option
 {
     Q_UNUSED(widget);
 
+    if (!m_match.isValid() && m_match.type() != Plasma::QueryMatch::InformationalMatch) {
+        return;
+    }
+
     bool oldClipping = painter->hasClipping();
     painter->setClipping(false);
 
@@ -327,50 +346,15 @@ void ResultItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option
     QRect iRect = QStyle::alignedRect(option->direction, Qt::AlignLeft, iconSize, contentsRect().toRect());
 
     painter->setRenderHint(QPainter::Antialiasing);
-    bool drawMixed = false;
 
-    if (hasFocus() || isSelected()) {
-        // here's what the next line means:
-        // we check to see if the scene has focus, but that's overridden by the mouse hovering an
-        // item ... or unless we are over 2 ticks into the higlight anim. complex but it works
-        if (((scene() && !scene()->views().isEmpty() && !scene()->views()[0]->hasFocus()) &&
-            !(option->state & QStyle::State_MouseOver)) || m_highlight > 2) {
-            drawIcon(painter, iRect, m_icon.pixmap(iconSize, QIcon::Active));
-        } else {
-            drawMixed = true;
-            ++m_highlight;
-
-            if (!m_highlightTimerId) {
-                m_highlightTimerId = startTimer(TIMER_INTERVAL);
-            }
-        }
-    } else if (m_highlight > 0) {
-        drawMixed = true;
-        --m_highlight;
-
-        if (!m_highlightTimerId) {
-            m_highlightTimerId = startTimer(TIMER_INTERVAL);
-        }
+    if (qFuzzyCompare(m_highlight + 1, 1)) {
+        drawIcon(painter, iRect, m_icon.pixmap(iconSize, QIcon::Disabled));
+    } else if (qFuzzyCompare(m_highlight, 1.0)) {
+        drawIcon(painter, iRect, m_icon.pixmap(iconSize, QIcon::Active));
     } else {
+        painter->setOpacity(painter->opacity() * (1 - m_highlight));
         drawIcon(painter, iRect, m_icon.pixmap(iconSize, QIcon::Disabled));
-    }
-
-    if (drawMixed) {
-        qreal factor = .2;
-
-        if (m_highlight == 1) {
-            factor = .4;
-        } else if (m_highlight == 2) {
-            factor = .6;
-        } else if (m_highlight > 2) {
-            factor = .8;
-        }
-
-        qreal activeOpacity = painter->opacity() * factor;
-
-        painter->setOpacity(painter->opacity() * (1 - factor));
-        drawIcon(painter, iRect, m_icon.pixmap(iconSize, QIcon::Disabled));
-        painter->setOpacity(activeOpacity);
+        painter->setOpacity(m_highlight);
         drawIcon(painter, iRect, m_icon.pixmap(iconSize, QIcon::Active));
         painter->setOpacity(1);
     }
@@ -439,34 +423,14 @@ void ResultItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option
     painter->setClipping(oldClipping);
 }
 
-void ResultItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *e)
-{
-    QGraphicsItem::hoverLeaveEvent(e);
-    //update();
-    //emit hoverLeave(this);
-    setFocus(Qt::MouseFocusReason);
-}
-
 void ResultItem::hoverEnterEvent(QGraphicsSceneHoverEvent *e)
 {
-    QGraphicsItem::hoverEnterEvent(e);
-    setFocus(Qt::MouseFocusReason);
-
-    if (scene()) {
-        scene()->clearSelection();
+    if (!m_sharedData->processHoverEvents || !m_match.isValid()) {
+        return;
     }
 
-    setSelected(true);
-}
-
-void ResultItem::timerEvent(QTimerEvent *e)
-{
-    Q_UNUSED(e)
-
-    killTimer(m_highlightTimerId);
-    m_highlightTimerId = 0;
-
-    update();
+    QGraphicsItem::hoverEnterEvent(e);
+    setFocus(Qt::MouseFocusReason);
 }
 
 void ResultItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
@@ -476,6 +440,19 @@ void ResultItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
     }
 }
 
+void ResultItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
+{
+    kDebug() << event->button() << Qt::LeftButton;
+
+    QMimeData * mime = m_runnerManager->mimeDataForMatch(m_match);
+
+    if (mime) {
+        QDrag * drag = new QDrag(event->widget());
+        drag->setMimeData(mime);
+        drag->exec();
+    }
+}
+
 bool ResultItem::mouseHovered() const
 {
     return m_mouseHovered;
@@ -495,24 +472,30 @@ void ResultItem::focusInEvent(QFocusEvent * event)
 
     setSelected(true);
     emit ensureVisibility(this);
-
-    if (!m_highlightTimerId) {
-        m_highlightTimerId = startTimer(TIMER_INTERVAL);
-    }
-
-    emit hoverEnter(this);
 }
 
-void ResultItem::focusOutEvent(QFocusEvent * event)
+void ResultItem::highlight(bool yes)
 {
-    QGraphicsWidget::focusOutEvent(event);
-    setZValue(0);
-
-    if (!m_highlightTimerId) {
-        m_highlightTimerId = startTimer(TIMER_INTERVAL);
+    if (yes) {
+        if (m_highlight < 1) {
+            m_highlightAnim->setDirection(QAbstractAnimation::Forward);
+            m_highlightAnim->start();
+        }
+    } else if (m_highlight > 0) {
+        m_highlightAnim->setDirection(QAbstractAnimation::Backward);
+        m_highlightAnim->start();
     }
+}
 
-    emit hoverLeave(this);
+qreal ResultItem::highlightState() const
+{
+    return m_highlight;
+}
+
+void ResultItem::setHighlightState(qreal highlight)
+{
+    m_highlight = highlight;
+    update();
 }
 
 void ResultItem::keyPressEvent(QKeyEvent *event)
@@ -528,11 +511,20 @@ QVariant ResultItem::itemChange(GraphicsItemChange change, const QVariant &value
 {
     if (change == QGraphicsItem::ItemSceneHasChanged) {
         calculateSize();
+    } else if (change == QGraphicsItem::ItemSelectedHasChanged) {
+        if (!isSelected()) {
+            m_highlightCheckTimer.start();
+        }
     }
 
     return QGraphicsWidget::itemChange(change, value);
 }
 
+void ResultItem::checkHighlighting()
+{
+    highlight(isSelected());
+}
+
 void ResultItem::resizeEvent(QGraphicsSceneResizeEvent *)
 {
     emit sizeChanged(this);
@@ -574,13 +566,12 @@ void ResultItem::showConfig()
 void ResultItem::calculateSize()
 {
     if (scene()) {
-        calculateSize(scene()->width(), scene()->height());
+        calculateSize(scene()->width());
     }
 }
 
-void ResultItem::calculateSize(int sceneWidth, int sceneHeight)
+void ResultItem::calculateSize(int sceneWidth)
 {
-    Q_UNUSED(sceneHeight)
     QRect textBounds(contentsRect().toRect());
 
     textBounds.setWidth(sceneWidth);
