@@ -22,6 +22,7 @@
 #include <KGlobalSettings>
 #include <KConfig>
 #include <KIconLoader>
+#include <KNotification>
 #include <kdesu/defaults.h>
 #include <KPasswordDialog>
 #include <KDebug>
@@ -46,6 +47,7 @@
 #include <QFrame>
 #include <QHBoxLayout>
 #include <QBoxLayout>
+#include <QSocketNotifier>
 #include <QTimerEvent>
 #include <QVBoxLayout>
 #include <QFile>
@@ -74,7 +76,6 @@
 #endif
 
 #define PASSDLG_HIDE_TIMEOUT 10000
-const int TIMEOUT_CODE = 2;
 
 //===========================================================================
 //
@@ -84,7 +85,8 @@ PasswordDlg::PasswordDlg(LockProcess *parent, GreeterPluginHandle *plugin, const
     : KDialog(parent, Qt::X11BypassWindowManagerHint),
       mPlugin( plugin ),
       mCapsLocked(-1),
-      mUnlockingFailed(false)
+      mUnlockingFailed(false),
+      sNot(0)
 {
     QWidget* w = mainWidget();
 
@@ -113,14 +115,14 @@ PasswordDlg::PasswordDlg(LockProcess *parent, GreeterPluginHandle *plugin, const
     cancel = new KPushButton( KStandardGuiItem::cancel(), w );
     mNewSessButton = new KPushButton( KGuiItem(i18n("Sw&itch User..."), "fork"), w );
 
-    // Using KXKB component
-    KPluginFactory *kxkbFactory = KPluginLoader("libkdeinit4_kxkb").factory();
+    // Using keyboard layout component
+    KPluginFactory *kxkbFactory = KPluginLoader("keyboard_layout_widget").factory();
     QWidget *kxkbComponent = NULL;
     if (kxkbFactory) {
         kxkbComponent = kxkbFactory->create<QWidget>(this);
     }
     else {
-        kDebug() << "can't load kxkb component library";
+        kDebug() << "can't load keyboard layout widget library";
     }
 
     QHBoxLayout *layStatus = new QHBoxLayout();
@@ -128,8 +130,11 @@ PasswordDlg::PasswordDlg(LockProcess *parent, GreeterPluginHandle *plugin, const
     layStatus->addWidget( mStatusLabel );
     layStatus->addStretch();
 
-    if( kxkbComponent )
+    if( kxkbComponent ) {
+	//TODO: without this the widget is off the parent area, but we need something better here
+        kxkbComponent->setFixedSize(48, 24);
         layStatus->addWidget( kxkbComponent, 0, Qt::AlignRight );
+    }
 
     QHBoxLayout *layButtons = new QHBoxLayout();
     layButtons->addWidget( mNewSessButton );
@@ -162,7 +167,7 @@ PasswordDlg::PasswordDlg(LockProcess *parent, GreeterPluginHandle *plugin, const
     connect(qApp, SIGNAL(activity()), SLOT(slotActivity()) );
 
     greet->start();
-    
+
     capsLocked();
 }
 
@@ -304,19 +309,26 @@ bool PasswordDlg::GRecvArr (char **ret)
         return false;
     *ret = buf;
     if (GRead (buf, len)) {
-	return true;
+        return true;
     } else {
-	::free(buf);
-	*ret = 0;
-	return false;
+        ::free(buf);
+        *ret = 0;
+        return false;
     }
 }
 
 void PasswordDlg::reapVerify()
 {
+    sNot->setEnabled( false );
+    sNot->deleteLater();
+    sNot = 0;
     ::close( sFd );
     int status;
-    ::waitpid( sPid, &status, 0 );
+    while (::waitpid( sPid, &status, 0 ) < 0)
+        if (errno != EINTR) { // This should not happen ...
+            cantCheck();
+            return;
+        }
     if (WIFEXITED(status))
         switch (WEXITSTATUS(status)) {
         case AuthOk:
@@ -331,6 +343,7 @@ void PasswordDlg::reapVerify()
             ok->setEnabled(false);
             cancel->setEnabled(false);
             mNewSessButton->setEnabled( false );
+            KNotification::event( "unlockfailed" );
             return;
         case AuthAbort:
             return;
@@ -343,7 +356,7 @@ void PasswordDlg::handleVerify()
     int ret;
     char *arr;
 
-    while (GRecvInt( &ret )) {
+    if (GRecvInt( &ret )) {
         switch (ret) {
         case ConvGetBinary:
             if (!GRecvArr( &arr ))
@@ -372,16 +385,15 @@ void PasswordDlg::handleVerify()
             if (!greet->textMessage( arr, false ))
                 static_cast< LockProcess* >(parent())->msgBox( this, QMessageBox::Information, QString::fromLocal8Bit( arr ) );
             ::free( arr );
-            continue;
+            return;
         case ConvPutError:
             if (!GRecvArr( &arr ))
                 break;
             if (!greet->textMessage( arr, true ))
                 static_cast< LockProcess* >(parent())->msgBox( this, QMessageBox::Warning, QString::fromLocal8Bit( arr ) );
             ::free( arr );
-            continue;
+            return;
         }
-        break;
     }
     reapVerify();
 }
@@ -393,7 +405,6 @@ void PasswordDlg::gplugReturnText( const char *text, int tag )
     GSendStr( text );
     if (text)
         GSendInt( tag );
-    handleVerify();
 }
 
 void PasswordDlg::gplugReturnBinary( const char *data )
@@ -407,7 +418,6 @@ void PasswordDlg::gplugReturnBinary( const char *data )
             GSendArr( len, data );
     } else
         GSendArr( 0, 0 );
-    handleVerify();
 }
 
 void PasswordDlg::gplugSetUser( const QString & )
@@ -420,7 +430,7 @@ void PasswordDlg::cantCheck()
     greet->failed();
     static_cast< LockProcess* >(parent())->msgBox( this, QMessageBox::Critical,
         i18n("Cannot unlock the session because the authentication system failed to work;\n"
-             "you must kill krunner_lock (pid %1) manually.", getpid()) );
+             "you must kill kscreenlocker (pid %1) manually.", getpid()) );
     greet->revive();
 }
 
@@ -433,6 +443,8 @@ void PasswordDlg::gplugStart()
     int sfd[2];
     char fdbuf[16];
 
+    if (sNot)
+        return;
     if (::socketpair(AF_LOCAL, SOCK_STREAM, 0, sfd)) {
         cantCheck();
         return;
@@ -448,17 +460,15 @@ void PasswordDlg::gplugStart()
         sprintf(fdbuf, "%d", sfd[1]);
         execlp(QFile::encodeName(KStandardDirs::findExe("kcheckpass")).data(),
                "kcheckpass",
-#ifdef HAVE_PAM
-               "-c", KSCREENSAVER_PAM_SERVICE,
-#endif
                "-m", mPlugin->info->method,
                "-S", fdbuf,
                (char *)0);
-        exit(20);
+        _exit(20);
     }
     ::close(sfd[1]);
     sFd = sfd[0];
-    handleVerify();
+    sNot = new QSocketNotifier(sFd, QSocketNotifier::Read, this);
+    connect(sNot, SIGNAL(activated(int)), SLOT(handleVerify()));
 }
 
 void PasswordDlg::gplugChanged()
@@ -489,7 +499,7 @@ void PasswordDlg::slotOK()
 void PasswordDlg::setVisible( bool visible )
 {
     QDialog::setVisible( visible );
-  
+
     if ( visible )
         QApplication::flush();
 }
@@ -507,7 +517,6 @@ void PasswordDlg::slotStartNewSession()
     KDialog *dialog = new KDialog( this, Qt::X11BypassWindowManagerHint );
     dialog->setModal( true );
     dialog->setButtons( KDialog::Yes | KDialog::No );
-    dialog->showButtonSeparator( true );
     dialog->setButtonGuiItem( KDialog::Yes, KGuiItem(i18n("&Start New Session"), "fork") );
     dialog->setButtonGuiItem( KDialog::No, KStandardGuiItem::cancel() );
     dialog->setDefaultButton( KDialog::Yes );
@@ -548,12 +557,12 @@ void PasswordDlg::slotStartNewSession()
 class LockListViewItem : public QTreeWidgetItem {
 public:
     LockListViewItem( QTreeWidget *parent,
-		      const QString &sess, const QString &loc, int _vt )
-	: QTreeWidgetItem( parent )
-	, vt( _vt )
+                      const QString &sess, const QString &loc, int _vt )
+        : QTreeWidgetItem( parent )
+        , vt( _vt )
     {
-	setText( 0, sess );
-	setText( 1, loc );
+        setText( 0, sess );
+        setText( 1, loc );
     }
 
     int vt;
