@@ -134,8 +134,8 @@ typedef struct {
   int priority;
 
   /** The i/o scheduling class and priority. */
-  int ioPriorityClass;  /** 0 for none, 1 for realtime, 2 for best-effort, 3 for idle.  -1 for error. */
-  int ioPriority;       /** Between 0 and 7.  0 is highest priority, 7 is lowest.  -1 for error. */
+  int ioPriorityClass;  /**< 0 for none, 1 for realtime, 2 for best-effort, 3 for idle.  -1 for error. */
+  int ioPriority;       /**< Between 0 and 7.  0 is highest priority, 7 is lowest.  -1 for error. */
 
   /**
     The total amount of virtual memory space that this process uses. This includes shared and
@@ -282,7 +282,10 @@ static bool getProcess( int pid, ProcessInfo *ps )
 
   /*There was a "(ps->vmRss+3) * sysconf(_SC_PAGESIZE)" here originally.  I have no idea why!  After comparing it to
   meminfo and other tools, this means we report the RSS by 12 bytes different compared to them.  So I'm removing the +3
-  to be consistent.  NEXT TIME COMMENT STRANGE THINGS LIKE THAT! :-)*/
+  to be consistent.  NEXT TIME COMMENT STRANGE THINGS LIKE THAT! :-)
+  
+    Update: I think I now know why.  The kernel reserves 3kb for process information.
+  */
   ps->vmRss = ps->vmRss * sysconf(_SC_PAGESIZE) / 1024; /*convert to KiB*/
   ps->vmSize /= 1024; /* convert to KiB */
 
@@ -454,6 +457,7 @@ void exitProcessList( void )
     removeCommand( "kill" );
     removeCommand( "setpriority" );
   }
+  closedir( procDir );
 
   exitPWUIDCache();
 }
@@ -488,7 +492,7 @@ void printProcessCountInfo( const char* cmd )
 
 void killProcess( const char* cmd )
 {
-  /* Sends a signal (not neccessarily kill!) to the process.  cmd is a string containing "kill <pid> <signal>" */
+  /* Sends a signal (not necessarily kill!) to the process.  cmd is a string containing "kill <pid> <signal>" */
   int sig, pid;
 
   sscanf( cmd, "%*s %d %d", &pid, &sig );
