@@ -329,7 +329,6 @@ void Client::updateDecoration( bool check_workspace_pos, bool force )
     if( !force &&
      (( decoration == NULL && noBorder() ) || ( decoration != NULL && !noBorder() )))
         return;
-    bool do_show = false;
     QRect oldgeom = geometry();
     blockGeometryUpdates( true );
     if( force )
@@ -350,7 +349,6 @@ void Client::updateDecoration( bool check_workspace_pos, bool force )
         XMoveWindow( display(), decoration->widget()->winId(), -padding_left, -padding_top );
         move( calculateGravitation( false ));
         plainResize( sizeForClientSize( clientSize()), ForceGeometrySet );
-        do_show = true;
         paintRedirector = new PaintRedirector( decoration->widget());
         connect( paintRedirector, SIGNAL( paintPending()), SLOT( repaintDecorationPending()));
         resizeDecorationPixmaps();
@@ -579,10 +577,18 @@ void Client::resizeDecorationPixmaps()
     QRect lr, rr, tr, br;
     layoutDecorationRects( lr, tr, rr, br, DecorationRelative );
 
-    decorationPixmapTop = QPixmap( tr.size() );
-    decorationPixmapBottom = QPixmap( br.size() );
-    decorationPixmapLeft = QPixmap( lr.size() );
-    decorationPixmapRight = QPixmap( rr.size() );
+    if ( decorationPixmapTop.size() != tr.size() )
+        decorationPixmapTop = QPixmap( tr.size() );
+
+    if ( decorationPixmapBottom.size() != br.size() )
+        decorationPixmapBottom = QPixmap( br.size() );
+
+    if ( decorationPixmapLeft.size() != lr.size() )
+        decorationPixmapLeft = QPixmap( lr.size() );
+
+    if ( decorationPixmapRight.size() != rr.size() )
+        decorationPixmapRight = QPixmap( rr.size() );
+
 #ifdef HAVE_XRENDER
     if ( Extensions::renderAvailable() ) {
         // Make sure the pixmaps are created with alpha channels
@@ -889,6 +895,9 @@ void Client::minimize( bool avoid_animation )
     if( effects && !avoid_animation ) // TODO: Shouldn't it tell effects at least about the change?
         static_cast<EffectsHandlerImpl*>(effects)->windowMinimized( effectWindow());
 
+    // when tiling, request a rearrangement
+    workspace()->notifyTilingWindowMinimizeToggled( this );
+
     // Update states of all other windows in this group
     if( clientGroup() )
         clientGroup()->updateStates( this );
@@ -905,9 +914,13 @@ void Client::unminimize( bool avoid_animation )
     updateAllowedActions();
     workspace()->updateMinimizedOfTransients( this );
     updateWindowRules();
+    workspace()->updateAllTiles();
     if( effects && !avoid_animation )
         static_cast<EffectsHandlerImpl*>( effects )->windowUnminimized( effectWindow() );
 
+    // when tiling, request a rearrangement
+    workspace()->notifyTilingWindowMinimizeToggled( this );
+
     // Update states of all other windows in this group
     if( clientGroup() )
         clientGroup()->updateStates( this );
@@ -968,7 +981,8 @@ void Client::setShade( ShadeMode mode )
     assert( decoration != NULL ); // noborder windows can't be shaded
     GeometryUpdatesBlocker blocker( this );
     // Decorations may turn off some borders when shaded
-    decoration->borders( border_left, border_right, border_top, border_bottom );
+    if( decoration )
+        decoration->borders( border_left, border_right, border_top, border_bottom );
 
     // TODO: All this unmapping, resizing etc. feels too much duplicated from elsewhere
     if ( isShade()) 
@@ -1012,7 +1026,8 @@ void Client::setShade( ShadeMode mode )
     updateVisibility();
     updateAllowedActions();
     workspace()->updateMinimizedOfTransients( this );
-    decoration->shadeChange();
+    if( decoration )
+        decoration->shadeChange();
     updateWindowRules();
 
     // Update states of all other windows in this group
@@ -1077,6 +1092,14 @@ void Client::updateVisibility()
             internalHide( Allowed );
         return;
         }
+    if( !isOnCurrentActivity())
+        {
+        if( compositing() && options->hiddenPreviews != HiddenPreviewsNever )
+            internalKeep( Allowed );
+        else
+            internalHide( Allowed );
+        return;
+        }
     bool belongs_to_desktop = false;
     for( ClientList::ConstIterator it = group()->members().constBegin();
         it != group()->members().constEnd();
@@ -1416,6 +1439,15 @@ void Client::setSkipPager( bool b )
     updateWindowRules();
     }
 
+void Client::setSkipSwitcher( bool set )
+    {
+    set = rules()->checkSkipSwitcher( set );
+    if( set == skipSwitcher() )
+        return;
+    skip_switcher = set;
+    updateWindowRules();
+    }
+
 void Client::setModal( bool m )
     { // Qt-3.2 can have even modal normal windows :(
     if( modal == m )
@@ -1454,6 +1486,66 @@ void Client::setDesktop( int desktop )
         clientGroup()->updateStates( this );
     }
 
+/**
+ * Sets whether the client is on @p activity.
+ * If you remove it from its last activity, then it's on all activities.
+ *
+ * Note: If it was on all activities and you try to remove it from one, nothing will happen;
+ * I don't think that's an important enough use case to handle here.
+ */
+void Client::setOnActivity( const QString &activity, bool enable )
+    {
+    if( activityList.contains(activity) == enable ) //nothing to do
+        return;
+    //check whether we should set it to all activities
+    QStringList newActivitiesList = activityList;
+    if (enable)
+        {
+        QStringList allActivities = KActivityConsumer().availableActivities();
+        if( !allActivities.contains(activity) ) //bogus ID
+            return;
+        newActivitiesList.append(activity);
+        }
+    else
+        newActivitiesList.removeOne(activity);
+    setOnActivities( newActivitiesList );
+    }
+
+/**
+ * set exactly which activities this client is on
+ */
+void Client::setOnActivities( QStringList newActivitiesList )
+    {
+    QStringList allActivities = KActivityConsumer().availableActivities();
+    if( newActivitiesList.size() == allActivities.size() || newActivitiesList.isEmpty() )
+        {
+        setOnAllActivities(true);
+        return;
+        }
+    activityList = newActivitiesList;
+    updateActivities( false );
+    }
+
+/**
+ * update after activities changed
+ */
+void Client::updateActivities( bool includeTransients )
+    {
+    /* FIXME do I need this?
+    if( decoration != NULL )
+        decoration->desktopChange();
+        */
+    if( includeTransients )
+        workspace()->updateOnAllActivitiesOfTransients( this );
+    workspace()->updateFocusChains( this, Workspace::FocusChainMakeFirst );
+    updateVisibility();
+    updateWindowRules();
+
+    // Update states of all other windows in this group
+    if( clientGroup() )
+        clientGroup()->updateStates( this );
+    }
+
 /**
  * Returns the virtual desktop within the workspace() the client window
  * is located in, 0 if it isn't located on any special desktop (not mapped yet),
@@ -1465,6 +1557,17 @@ int Client::desktop() const
     return desk;
     }
 
+/**
+ * Returns the list of activities the client window is on.
+ * if it's on all activities, the list will be empty.
+ * Don't use this, use isOnActivity() and friends (from class Toplevel)
+ * FIXME do I need to consider if it's not mapped yet?
+ */
+QStringList Client::activities() const
+    {
+    return activityList;
+    }
+
 void Client::setOnAllDesktops( bool b )
     {
     if(( b && isOnAllDesktops() ) ||
@@ -1480,6 +1583,26 @@ void Client::setOnAllDesktops( bool b )
         clientGroup()->updateStates( this );
     }
 
+/**
+ * if @p on is true, sets on all activities.
+ * if it's false, sets it to only be on the current activity
+ */
+void Client::setOnAllActivities( bool on )
+    {
+    if( on == isOnAllActivities() )
+        return;
+    if( on )
+        {
+        activityList.clear();
+        updateActivities( true );
+        }
+    else
+        {
+        setOnActivity(Workspace::self()->currentActivity(), true);
+        workspace()->updateOnAllActivitiesOfTransients( this );
+        }
+    }
+
 /**
  * Performs activation and/or raising of the window
  */
@@ -1757,7 +1880,7 @@ void Client::getMotifHints()
         updateDecoration( true ); // Check if noborder state has changed
     }
 
-void Client::readIcons( Window win, QPixmap* icon, QPixmap* miniicon )
+void Client::readIcons( Window win, QPixmap* icon, QPixmap* miniicon, QPixmap* bigicon, QPixmap* hugeicon )
     {
     // Get the icons, allow scaling
     if( icon != NULL )
@@ -1769,16 +1892,32 @@ void Client::readIcons( Window win, QPixmap* icon, QPixmap* miniicon )
         else
             *miniicon = QPixmap();
         }
+    if( bigicon != NULL )
+        {
+        if( icon == NULL || !icon->isNull() )
+            *bigicon = KWindowSystem::icon( win, 64, 64, false, KWindowSystem::NETWM | KWindowSystem::WMHints );
+        else
+            *bigicon = QPixmap();
+        }
+    if( hugeicon != NULL )
+        {
+        if( icon == NULL || !icon->isNull() )
+            *hugeicon = KWindowSystem::icon( win, 128, 128, false, KWindowSystem::NETWM | KWindowSystem::WMHints );
+        else
+            *hugeicon = QPixmap();
+        }
     }
 
 void Client::getIcons()
     {
     // First read icons from the window itself
-    readIcons( window(), &icon_pix, &miniicon_pix );
+    readIcons( window(), &icon_pix, &miniicon_pix, &bigicon_pix, &hugeicon_pix );
     if( icon_pix.isNull() )
         { // Then try window group
         icon_pix = group()->icon();
         miniicon_pix = group()->miniIcon();
+        bigicon_pix = group()->bigIcon();
+        hugeicon_pix = group()->hugeIcon();
         }
     if( icon_pix.isNull() && isTransient() )
         { // Then mainclients
@@ -1789,17 +1928,34 @@ void Client::getIcons()
             {
             icon_pix = (*it)->icon();
             miniicon_pix = (*it)->miniIcon();
+            bigicon_pix = (*it)->bigIcon();
+            hugeicon_pix = (*it)->hugeIcon();
             }
         }
     if( icon_pix.isNull())
         { // And if nothing else, load icon from classhint or xapp icon
         icon_pix = KWindowSystem::icon( window(), 32, 32, true, KWindowSystem::ClassHint | KWindowSystem::XApp );
         miniicon_pix = KWindowSystem::icon( window(), 16, 16, true, KWindowSystem::ClassHint | KWindowSystem::XApp );
+        bigicon_pix = KWindowSystem::icon( window(), 64, 64, false, KWindowSystem::ClassHint | KWindowSystem::XApp );
+        hugeicon_pix = KWindowSystem::icon( window(), 128, 128, false, KWindowSystem::ClassHint | KWindowSystem::XApp );
         }
     if( isManaged() && decoration != NULL )
         decoration->iconChange();
     }
 
+QPixmap Client::icon( const QSize& size ) const
+    {
+    const int iconSize = qMin( size.width(), size.height() );
+    if( iconSize <= 16 )
+        return miniIcon();
+    else if( iconSize <= 32 )
+        return icon();
+    if( iconSize <= 64 )
+        return bigIcon();
+    else
+        return hugeIcon();
+    }
+
 void Client::getWindowProtocols()
     {
     Atom* p;
