@@ -134,7 +134,10 @@ class Client
         QSize adjustedSize() const;
 
         QPixmap icon() const;
+        QPixmap icon( const QSize& size ) const;
         QPixmap miniIcon() const;
+        QPixmap bigIcon() const;
+        QPixmap hugeIcon() const;
 
         bool isActive() const;
         void setActive( bool );
@@ -143,6 +146,12 @@ class Client
         void setDesktop( int );
         void setOnAllDesktops( bool set );
 
+        virtual QStringList activities() const;
+        void setOnActivity( const QString &activity, bool enable );
+        void setOnAllActivities( bool set );
+        void setOnActivities( QStringList newActivitiesList );
+        void updateActivities( bool includeTransients );
+
         /// Is not minimized and not hidden. I.e. normally visible on some virtual desktop.
         bool isShown( bool shaded_is_shown ) const;
         bool isHiddenInternal() const; // For compositing
@@ -179,6 +188,9 @@ class Client
         bool skipPager() const;
         void setSkipPager( bool );
 
+        bool skipSwitcher() const;
+        void setSkipSwitcher( bool set );
+
         bool keepAbove() const;
         void setKeepAbove( bool );
         bool keepBelow() const;
@@ -290,7 +302,7 @@ class Client
 
         static bool belongToSameApplication( const Client* c1, const Client* c2, bool active_hack = false );
         static bool sameAppWindowRoleMatch( const Client* c1, const Client* c2, bool active_hack );
-        static void readIcons( Window win, QPixmap* icon, QPixmap* miniicon );
+        static void readIcons( Window win, QPixmap* icon, QPixmap* miniicon, QPixmap* bigicon, QPixmap* hugeicon );
 
         void minimize( bool avoid_animation = false );
         void unminimize( bool avoid_animation = false );
@@ -503,6 +515,7 @@ class Client
         KDecoration* decoration;
         Bridge* bridge;
         int desk;
+        QStringList activityList;
         bool buttonDown;
         bool moveResizeMode;
         bool move_faked_activity;
@@ -556,6 +569,7 @@ class Client
         uint Pping : 1; ///< Does it support _NET_WM_PING?
         uint input : 1; ///< Does the window want input in its wm_hints
         uint skip_pager : 1;
+        uint skip_switcher : 1;
         uint motif_may_resize : 1;
         uint motif_may_move : 1;
         uint motif_may_close : 1;
@@ -575,6 +589,8 @@ class Client
         void getWindowProtocols();
         QPixmap icon_pix;
         QPixmap miniicon_pix;
+        QPixmap bigicon_pix;
+        QPixmap hugeicon_pix;
         QCursor cursor;
         // DON'T reorder - Saved to config files !!!
         enum FullScreenMode
@@ -780,6 +796,16 @@ inline QPixmap Client::miniIcon() const
     return miniicon_pix;
     }
 
+inline QPixmap Client::bigIcon() const
+    {
+    return bigicon_pix;
+    }
+
+inline QPixmap Client::hugeIcon() const
+    {
+    return hugeicon_pix;
+    }
+
 inline QRect Client::geometryRestore() const
     {
     return geom_restore;
@@ -805,6 +831,11 @@ inline bool Client::skipPager() const
     return skip_pager;
     }
 
+inline bool Client::skipSwitcher() const
+    {
+    return skip_switcher;
+    }
+
 inline bool Client::keepAbove() const
     {
     return keep_above;
