@@ -28,12 +28,13 @@
 // IN THE SOFTWARE.
 //////////////////////////////////////////////////////////////////////////////
 
-#include "oxygen.h"
+#include "oxygenanimation.h"
 #include "oxygenclientgroupitemdata.h"
 #include "oxygenconfiguration.h"
+#include "oxygendecohelper.h"
+#include "oxygenfactory.h"
+#include "oxygenshadowcache.h"
 #include "oxygentitleanimationdata.h"
-#include "lib/oxygenanimation.h"
-#include "lib/helper.h"
 
 #include <kcommondecoration.h>
 #include <QtCore/QBasicTimer>
@@ -43,8 +44,8 @@
 namespace Oxygen
 {
 
-    class OxygenSizeGrip;
-    class OxygenClient : public KCommonDecorationUnstable
+    class SizeGrip;
+    class Client : public KCommonDecorationUnstable
     {
 
         Q_OBJECT
@@ -56,10 +57,10 @@ namespace Oxygen
         public:
 
         //! constructor
-        OxygenClient(KDecorationBridge *b, OxygenFactory *f);
+        Client(KDecorationBridge *b, Factory *f);
 
         //! destructor
-        virtual ~OxygenClient();
+        virtual ~Client();
 
         //! decoration name
         virtual QString visibleName() const;
@@ -112,7 +113,7 @@ namespace Oxygen
         virtual void reset( unsigned long changed );
 
         //! return associated configuration
-        const OxygenConfiguration& configuration( void ) const
+        const Configuration& configuration( void ) const
         { return configuration_; }
 
         //!@name glow animation
@@ -140,11 +141,11 @@ namespace Oxygen
         //@}
 
         //! helper class
-        OxygenHelper& helper( void ) const
+        DecoHelper& helper( void ) const
         { return factory_->helper(); }
 
         //! helper class
-        OxygenShadowCache& shadowCache( void ) const
+        ShadowCache& shadowCache( void ) const
         { return factory_->shadowCache(); }
 
         //!@name metrics and color definitions
@@ -206,6 +207,18 @@ namespace Oxygen
 
         protected:
 
+        //! return shadow cache key associated to this client
+        ShadowCache::Key key( void ) const
+        {
+            ShadowCache::Key key;
+            key.active = isActive() || isForcedActive();
+            key.useOxygenShadows = configuration().useOxygenShadows();
+            key.isShade = isShade();
+            key.hasTitleOutline = configuration().drawTitleOutline();
+            key.hasBorder = ( configuration().frameBorder() > Configuration::BorderNone );
+            return key;
+        }
+
         //! true when decoration is forced active
         void setForceActive( bool value )
         { forceActive_ = value; }
@@ -284,7 +297,7 @@ namespace Oxygen
         //@}
 
         //! close tab matching give button
-        virtual bool closeItem( const OxygenButton* );
+        virtual bool closeItem( const Button* );
 
         //! index of item matching point
         int itemClicked( const QPoint& position, bool between = false ) const
@@ -339,7 +352,7 @@ namespace Oxygen
         { return (bool)sizeGrip_; }
 
         //! size grip
-        OxygenSizeGrip& sizeGrip( void ) const
+        SizeGrip& sizeGrip( void ) const
         { return *sizeGrip_; }
 
         //@}
@@ -360,13 +373,13 @@ namespace Oxygen
         private:
 
         //! factory
-        OxygenFactory* factory_;
+        Factory* factory_;
 
         //! size grip widget
-        OxygenSizeGrip* sizeGrip_;
+        SizeGrip* sizeGrip_;
 
         //! configuration
-        OxygenConfiguration configuration_;
+        Configuration configuration_;
 
         //! glow animation
         Animation::Pointer glowAnimation_;
