@@ -105,7 +105,13 @@ public:
         NoBorderOp,
         NoOp,
         SetupWindowShortcutOp,
-        ApplicationRulesOp
+        ApplicationRulesOp,
+        RemoveClientFromGroupOp, // Remove from group
+        CloseClientGroupOp, // Close the group
+        MoveClientInGroupLeftOp, // Move left in the group
+        MoveClientInGroupRightOp, // Move right in the group
+        ToggleClientTiledStateOp, // put a floating client into tiling
+        ClientGroupDragOp
         };
     /**
      * Basic color types that should be recognized by all decoration styles.
@@ -129,12 +135,13 @@ public:
      */
     enum
         {
-        SettingDecoration = 1 << 0, ///< The decoration was changed
-        SettingColors     = 1 << 1, ///< The color palette was changed
-        SettingFont       = 1 << 2, ///< The titlebar font was changed
-        SettingButtons    = 1 << 3, ///< The button layout was changed
-        SettingTooltips   = 1 << 4, ///< The tooltip setting was changed
-        SettingBorder     = 1 << 5  ///< The border size setting was changed
+        SettingDecoration  = 1 << 0, ///< The decoration was changed
+        SettingColors      = 1 << 1, ///< The color palette was changed
+        SettingFont        = 1 << 2, ///< The titlebar font was changed
+        SettingButtons     = 1 << 3, ///< The button layout was changed
+        SettingTooltips    = 1 << 4, ///< The tooltip setting was changed
+        SettingBorder      = 1 << 5, ///< The border size setting was changed
+        SettingCompositing = 1 << 6  ///< Compositing settings was changed
         };
 
     /**
@@ -185,24 +192,50 @@ public:
         AbilityColorButtonFore = 2021, ///< decoration supports button foreground color
         ABILITYCOLOR_END, ///< @internal
         // compositing
-        AbilityCompositingShadow = 3000, ///< decoration supports window shadows
+        AbilityProvidesShadow = 3000, ///< The decoration draws its own shadows.
+                                      ///  @since 4.3
+        AbilityUsesAlphaChannel = 3001, ///< The decoration isn't clipped to the mask when compositing is enabled.
+                                        ///  The mask is still used to define the input region and the blurred
+                                        ///  region, when the blur plugin is enabled.
+                                        ///  @since 4.3
+        AbilityExtendIntoClientArea = 3002, ///< The decoration respects transparentRect()
+                                            ///  @since 4.4
+        // Tabbing
+        AbilityClientGrouping = 4000, ///< The decoration supports tabbing
         // TODO colors for individual button types
         ABILITY_DUMMY = 10000000
         };
 
     enum Requirement { REQUIREMENT_DUMMY = 1000000 };
+    
+    /**
+    *
+    * Returns mimeType used to drag and drop clientGroupItems 
+    */
+    
+    static QString clientGroupItemDragMimeType();
+
 };
 
-/**
- * Decoration shadow type.
- */
-enum ShadowType
+class KWIN_EXPORT ClientGroupItem
     {
-    ShadowBorderedActive = 0, ///< Active shadow of decorated windows
-    ShadowBorderedInactive, ///< Inctive shadow of decorated windows
-    ShadowBorderlessActive, ///< Active shadow of undecorated windows
-    ShadowBorderlessInactive, ///< Inctive shadow of undecorated windows
-    ShadowOther ///< Shadow of all other windows (Menus, comboboxes, tooltips, etc.)
+    public:
+        ClientGroupItem( QString t, QIcon i )
+            {
+            title_ = t;
+            icon_ = i;
+            }
+        inline QIcon icon() const
+            {
+            return icon_;
+            }
+        inline QString title() const
+            {
+            return title_;
+            }
+    private:
+        QString title_;
+        QIcon icon_;
     };
 
 class KDecorationProvides
@@ -294,7 +327,7 @@ public:
      * in titleButtonsLeft().
      *
      * The default ( which is also returned if customButtonPositions returns false )
-     * is "HIAX".
+     * is "HIA__X".
      * Unknown buttons in the returned string must be ignored.
      * The changed flags for this setting is SettingButtons.
      */
@@ -598,6 +631,20 @@ class KWIN_EXPORT KDecoration
          * Convenience function that returns the height of the decoration.
          */
         int height() const;  // convenience
+        /**
+         * Returns the rectangle within the window frame that should be transparent.
+         * Usually this rectangle is the same as the client area, and it is never
+         * larger.
+         *
+         * If the client has requested that the window frame is extended into the
+         * client area, this rectangle is smaller than the client area.
+         *
+         * If the window frame should cover the whole client area, a null rectangle
+         * is returned.
+         *
+         * @since 4.4
+         */
+        QRect transparentRect() const;
         /**
          * This function is the default handler for mouse events. All mouse events
          * that are not handled by the decoration itself should be passed to it
@@ -678,6 +725,7 @@ class KWIN_EXPORT KDecoration
          * isShade() to get the current state.
          */
         virtual void shadeChange() = 0;
+
     Q_SIGNALS:
         /**
          * This signal is emitted whenever the window's keep-above state changes.
@@ -687,6 +735,7 @@ class KWIN_EXPORT KDecoration
          * This signal is emitted whenever the window's keep-below state changes.
          */
         void keepBelowChanged( bool );
+
     public:
         /**
          * This function may be reimplemented to provide custom bound drawing
@@ -854,6 +903,7 @@ class KWIN_EXPORT KDecoration
          * @internal
          */
         void emitKeepBelowChanged( bool below );
+
     private:
         KDecorationBridge* bridge_;
         QWidget* w_;
@@ -862,56 +912,96 @@ class KWIN_EXPORT KDecoration
         friend class KDecorationUnstable; // for bridge_
         static KDecorationOptions* options_;
         KDecorationPrivate* d;
+
     };
 
 /**
- * @warning THIS CLASS IS UNSTABLE AND WILL ONLY BE SUPPORTED IN KDE 4.2!
+ * @warning THIS CLASS IS UNSTABLE!
  */
 class KWIN_EXPORT KDecorationUnstable
     : public KDecoration
     {
     Q_OBJECT
+
     public:
         KDecorationUnstable( KDecorationBridge* bridge, KDecorationFactory* factory );
         virtual ~KDecorationUnstable();
-
         /**
-         * This function should return the positions of the shadow quads to be rendered.
-         * All positions are relative to the window's top-left corner. Only "bordered"
-         * windows will call this method.
+         * This function can return additional padding values that are added outside the
+         * borders of the window, and can be used by the decoration if it wants to paint
+         * outside the frame.
+         *
+         * The typical use case is for drawing a drop shadow or glowing effect around the window.
+         *
+         * The area outside the frame cannot receive input, and when compositing is disabled,
+         * painting is clipped to the mask, or the window frame if no mask is defined.
          */
-        virtual QList<QRect> shadowQuads( ShadowType type ) const;
+        virtual void padding(int &left, int &right, int &top, int &bottom) const;
         /**
-         * This function should return the desired opacity of the shadow.
-         */
-        virtual double shadowOpacity( ShadowType type ) const;
-        /**
-         * This function should return the desired brightness of the shadow.
-         */
-        virtual double shadowBrightness( ShadowType type ) const;
-        /**
-         * This function should return the desired saturation of the shadow.
-         */
-        virtual double shadowSaturation( ShadowType type ) const;
-
-        /**
-         * Force a repaint of the shadow. Automatically called when the window changes states.
-         */
-        void repaintShadow();
-        /**
-         * Returns @a true if compositing is enabled (Currently useless to decorations,
-         * use \a shadowsActive() instead).
+         * Returns @a true if compositing--and therefore ARGB--is enabled.
          */
         bool compositingActive() const;
+
+        // Window tabbing
+
         /**
-         * Returns @a true if compositing is enabled and the shadow effect is activated
-         * by the current user.
+         * Returns whether or not this client group contains the active client.
          */
-        bool shadowsActive() const;
+        bool isClientGroupActive();
         /**
-         * Returns the opacity that the decoration will be rendered at.
+         * Return a list of all the clients in the group that contains the client that this
+         * decoration is attached to.
          */
-        double opacity() const;
+        QList< ClientGroupItem > clientGroupItems() const;
+        /**
+         * Returns a unique identifier for the client at index \p index of the client group list.
+         * \see moveItemToClientGroup()
+         */
+        long itemId( int index );
+        /**
+         * Returns the list index of the currently visible client in this group.
+         */
+        int visibleClientGroupItem();
+        /**
+         * Switch the currently visible client to the one at list index \p index.
+         */
+        void setVisibleClientGroupItem( int index );
+        /**
+         * Move the client at index \p index to the position before the client at index \p before.
+         */
+        void moveItemInClientGroup( int index, int before );
+        /**
+         * Move the client that's unique identifier is \p itemId to the position before the client
+         * at index \p before if set, otherwise the end of the list. This call is to move clients
+         * between two different groups, if moving in the same group then use
+         * moveItemInClientGroup() instead.
+         * \see itemId()
+         */
+        void moveItemToClientGroup( long itemId, int before = -1 );
+        /**
+         * Remove the client at index \p index from the group. If \p newGeom is set then the client
+         * will move and resize to the specified geometry, otherwise it will stay where the group
+         * is located.
+         */
+        void removeFromClientGroup( int index, const QRect& newGeom = QRect() );
+        /**
+         * Close the client at index \p index.
+         */
+        void closeClientGroupItem( int index );
+        /**
+         * Close all windows in this group.
+         */
+        void closeAllInClientGroup();
+        /**
+         * Display the right-click client menu belonging to the client at index \p index at the
+         * global coordinates specified by \p pos.
+         */
+        void displayClientMenu( int index, const QPoint& pos );
+        /**
+         * Determine which action the user has mapped \p button to. Useful for determining whether
+         * a button press was for window tab dragging or for displaying the client menu.
+         */
+        WindowOperation buttonToWindowOperation( Qt::MouseButtons button );
     };
 
 inline
