@@ -39,14 +39,17 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <kxerrorhandler.h>
 #include <kdefakes.h>
 #include <QtDBus/QtDBus>
-
+#include <stdlib.h>
 #include <kdialog.h>
 #include <kstandarddirs.h>
 #include <kdebug.h>
+#include <kde_file.h>
 #include <QLabel>
-#include <QComboBox>
+#include <KComboBox>
 #include <QVBoxLayout>
 
+#include <ksmserver_interface.h>
+
 #include "atoms.h"
 #include "options.h"
 #include "sm.h"
@@ -68,7 +71,7 @@ Atoms* atoms;
 
 int screen_number = -1;
 
-static bool initting = false;
+bool initting = false;
 
 /**
  * Whether to run Xlib in synchronous mode and print backtraces for X errors.
@@ -77,6 +80,9 @@ static bool initting = false;
  */
 static bool kwin_sync = false;
 
+// errorMessage is only used ifndef NDEBUG, and only in one place.
+// it might be worth reevaluating why this is used? I don't know.
+#ifndef NDEBUG
 /**
  * Outputs: "Error: <error> (<value>), Request: <request>(<value>), Resource: <value>"
  */
@@ -161,9 +167,11 @@ static QByteArray errorMessage( const XErrorEvent& event, Display* dpy )
         }
     return ret;
     }
+#endif
 
 static int x11ErrorHandler( Display* d, XErrorEvent* e )
     {
+    Q_UNUSED( d );
     bool ignore_badwindow = true; // Might be temporary
 
     if( initting && ( e->request_code == X_ChangeWindowAttributes || e->request_code == X_GrabKey ) &&
@@ -176,17 +184,14 @@ static int x11ErrorHandler( Display* d, XErrorEvent* e )
     if( ignore_badwindow && ( e->error_code == BadWindow || e->error_code == BadColor ))
         return 0;
 
+#ifndef NDEBUG
     //fprintf( stderr, "kwin: X Error (%s)\n", KXErrorHandler::errorMessage( *e, d ).data());
     kWarning( 1212 ) << "kwin: X Error (" << errorMessage( *e, d ) << ")";
+#endif
 
     if( kwin_sync )
-    fprintf( stderr, "%s\n", kBacktrace().toLocal8Bit().data() );
+        fprintf( stderr, "%s\n", kBacktrace().toLocal8Bit().data() );
 
-    if( initting )
-        {
-        fputs( i18n( "kwin: failure during initialization; aborting").toLocal8Bit(), stderr );
-        exit( 1 );
-        }
     return 0;
     }
 
@@ -206,7 +211,7 @@ class AlternativeWMDialog : public KDialog
                 "You can select another window manager to run:" );
             QLabel* textLabel = new QLabel( text, mainWidget );
             layout->addWidget( textLabel );
-            wmList = new QComboBox( mainWidget );
+            wmList = new KComboBox( mainWidget );
             wmList->setEditable( true );
             layout->addWidget( wmList );
 
@@ -231,7 +236,7 @@ class AlternativeWMDialog : public KDialog
             { return wmList->currentText(); }
 
     private:
-        QComboBox* wmList;
+        KComboBox* wmList;
 };
 
 int Application::crashes = 0;
@@ -246,13 +251,12 @@ Application::Application()
         XSynchronize( display(), True );
         kDebug( 1212 ) << "Running KWin in sync mode";
         }
+    setQuitOnLastWindowClosed( false );
     KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
     KSharedConfig::Ptr config = KGlobal::config();
     if( !config->isImmutable() && args->isSet( "lock" ))
         {
-#ifdef __GNUC__
-#warning this shouldn not be necessary
-#endif
+        // TODO: This shouldn't be necessary
         //config->setReadOnly( true );
         config->reparseConfiguration();
         }
@@ -311,7 +315,7 @@ Application::Application()
 
     atoms = new Atoms;
 
-    initting = false; // TODO
+//    initting = false; // TODO
 
     // This tries to detect compositing options and can use GLX. GLX problems
     // (X errors) shouldn't cause kwin to abort, so this is out of the
@@ -379,7 +383,8 @@ void Application::crashHandler( int signal )
 
     fprintf( stderr, "Application::crashHandler() called with signal %d; recent crashes: %d\n", signal, crashes );
     char cmd[1024];
-    sprintf( cmd, "kwin --crashes %d &", crashes );
+    sprintf( cmd, "%s --crashes %d &",
+            QFile::encodeName(QCoreApplication::applicationFilePath()).constData(), crashes );
 
     sleep( 1 );
     system( cmd );
@@ -408,11 +413,15 @@ KDE_EXPORT int kdemain( int argc, char * argv[] )
             }
         }
 
+    // KWin only works properly with Qt's native X11 backend; override any compile-time
+    // or command line settings to raster or OpenGL.
+    QApplication::setGraphicsSystem("native");
+
     if( !restored )
         { // We only do the multihead fork if we are not restored by the session
           // manager, since the session manager will register multiple kwins,
           // one for each screen...
-        QByteArray multiHead = getenv( "KDE_MULTIHEAD" );
+        QByteArray multiHead = qgetenv( "KDE_MULTIHEAD" );
         if( multiHead.toLower() == "true" )
             {
             Display* dpy = XOpenDisplay( NULL );
@@ -481,28 +490,31 @@ KDE_EXPORT int kdemain( int argc, char * argv[] )
     args.add( "replace", ki18n( "Replace already-running ICCCM2.0-compliant window manager" ));
     args.add( "crashes <n>", ki18n( "Indicate that KWin has recently crashed n times" ));
     KCmdLineArgs::addCmdLineOptions( args );
-
-    if( signal( SIGTERM, KWin::sighandler ) == SIG_IGN )
-        signal( SIGTERM, SIG_IGN );
-    if( signal( SIGINT, KWin::sighandler ) == SIG_IGN )
-        signal( SIGINT, SIG_IGN );
-    if( signal( SIGHUP, KWin::sighandler ) == SIG_IGN )
-        signal( SIGHUP, SIG_IGN );
-
-    // HACK: This is needed for AIGLX
-    if( qstrcmp( getenv( "KWIN_DIRECT_GL" ), "1" ) != 0 )
-        setenv( "LIBGL_ALWAYS_INDIRECT","1", true );
+    
+    if( KDE_signal( SIGTERM, KWin::sighandler ) == SIG_IGN )
+        KDE_signal( SIGTERM, SIG_IGN );
+    if( KDE_signal( SIGINT, KWin::sighandler ) == SIG_IGN )
+        KDE_signal( SIGINT, SIG_IGN );
+    if( KDE_signal( SIGHUP, KWin::sighandler ) == SIG_IGN )
+        KDE_signal( SIGHUP, SIG_IGN );
 
     // HACK: this is needed to work around a Qt4.4.0RC1 bug (#157659)
     setenv( "QT_SLOW_TOPLEVEL_RESIZE", "1", true );
 
+    // Disable the glib event loop integration, since it seems to be responsible
+    // for several bug reports about high CPU usage (bug #239963)
+    setenv( "QT_NO_GLIB", "1", true );
+
+    org::kde::KSMServerInterface ksmserver( "org.kde.ksmserver", "/KSMServer", QDBusConnection::sessionBus());
+    ksmserver.suspendStartup( "kwin" );
     KWin::Application a;
+    ksmserver.resumeStartup( "kwin" );
     KWin::SessionManager weAreIndeed;
     KWin::SessionSaveDoneHelper helper;
     KGlobal::locale()->insertCatalog( "kwin_effects" );
 
     // Announce when KWIN_DIRECT_GL is set for above HACK
-    if( qstrcmp( getenv( "KWIN_DIRECT_GL" ), "1" ) == 0 )
+    if( qstrcmp( qgetenv( "KWIN_DIRECT_GL" ), "1" ) == 0 )
         kDebug( 1212 ) << "KWIN_DIRECT_GL set, not forcing LIBGL_ALWAYS_INDIRECT=1";
 
     fcntl( XConnectionNumber( KWin::display() ), F_SETFD, 1 );
@@ -517,6 +529,6 @@ KDE_EXPORT int kdemain( int argc, char * argv[] )
         appname, QDBusConnectionInterface::DontQueueService );
 
     return a.exec();
-    }
+}
 
 #include "main.moc"
