@@ -156,9 +156,7 @@ NET::WindowType Toplevel::windowType( bool direct, int supported_types ) const
     if( direct )
         return wt;
     const Client* cl = dynamic_cast< const Client* >( this );
-    // TODO
-    //NET::WindowType wt2 = rules()->checkType( wt );
-    NET::WindowType wt2 = wt;
+    NET::WindowType wt2 = cl ? cl->rules()->checkType( wt ) : wt;
     if( wt != wt2 )
         {
         wt = wt2;
