@@ -151,6 +151,7 @@ void AbstractTaskItem::setPreferredOnscreenSize()
 
 AbstractTaskItem::~AbstractTaskItem()
 {
+    stopWindowHoverEffect();
     emit destroyed(this);
     Plasma::ToolTipManager::self()->unregisterWidget(this);
 }
@@ -200,16 +201,19 @@ void AbstractTaskItem::setTaskFlags(const TaskFlags flags)
 {
     if (((m_flags & TaskWantsAttention) != 0) != ((flags & TaskWantsAttention) != 0)) {
         //kDebug() << "task attention state changed" << m_attentionTimerId;
+        m_flags = flags;
         if (flags & TaskWantsAttention) {
-            m_applet->needsVisualFocus();
+            m_applet->needsVisualFocus(true);
             // start attention getting
             if (!m_attentionTimerId) {
                 m_attentionTimerId = startTimer(500);
             }
-        } else if (m_attentionTimerId) {
-            killTimer(m_attentionTimerId);
-            m_attentionTimerId = 0;
-            // stop attention getting
+        } else {
+            m_applet->needsVisualFocus(false);
+            if (m_attentionTimerId) {
+                killTimer(m_attentionTimerId);
+                m_attentionTimerId = 0;
+            }
         }
     }
 
@@ -367,10 +371,6 @@ void AbstractTaskItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
 
 void AbstractTaskItem::stopWindowHoverEffect()
 {
-    if (!parentGroup()) {
-        return;
-    }
-
     if (m_hoverEffectTimerId) {
         killTimer(m_hoverEffectTimerId);
         m_hoverEffectTimerId = 0;
@@ -383,7 +383,7 @@ void AbstractTaskItem::stopWindowHoverEffect()
 
 void AbstractTaskItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 {
-    if (event->button() == Qt::LeftButton) {
+    if (event->button() == Qt::LeftButton && boundingRect().contains(event->pos())) {
         activate();
     }
 }
@@ -528,6 +528,7 @@ void AbstractTaskItem::paint(QPainter *painter,
 
 void AbstractTaskItem::syncActiveRect()
 {
+    m_cachedShadow = QPixmap();
     Plasma::FrameSvg *itemBackground = m_applet->itemBackground();
     itemBackground->setElementPrefix("normal");
 
@@ -547,12 +548,11 @@ void AbstractTaskItem::syncActiveRect()
     // check to see if there is enough room!
     QFontMetrics fm(font());
     const int minimumWidth = left + 8 + IconTextSpacing + right;
-    m_showText = (size().width() >= fm.width("mmm") + minimumWidth);
+    m_showText = (size().width() >= fm.width("M") * 6 + minimumWidth);
 }
 
 void AbstractTaskItem::resizeEvent(QGraphicsSceneResizeEvent *event)
 {
-    m_cachedShadow = QPixmap();
     syncActiveRect();
     resizeBackground(event->newSize().toSize());
 }
@@ -589,14 +589,14 @@ void AbstractTaskItem::drawBackground(QPainter *painter, const QStyleOptionGraph
     */
     Plasma::FrameSvg *itemBackground = m_applet->itemBackground();
 
-    //since a single framesvg is shared between all tasks, we could have to resize it even if there wasn't a resizeevent
-    if (size().toSize() != itemBackground->frameSize()) {
-        resizeBackground(size().toSize());
-    }
-
     if (~option->state & QStyle::State_Sunken && 
         (!m_backgroundFadeAnim || m_backgroundFadeAnim->state() != QAbstractAnimation::Running)) {
         itemBackground->setElementPrefix(m_backgroundPrefix);
+        //since a single framesvg is shared between all tasks, we could have to resize it even if there wasn't a resizeevent
+        if (size().toSize() != itemBackground->frameSize()) {
+            resizeBackground(size().toSize());
+        }
+
         if (itemBackground->frameSize() == m_activeRect.size().toSize()) {
             itemBackground->paintFrame(painter, m_activeRect.topLeft());
         } else {
@@ -607,6 +607,11 @@ void AbstractTaskItem::drawBackground(QPainter *painter, const QStyleOptionGraph
     }
 
     itemBackground->setElementPrefix(m_oldBackgroundPrefix);
+    //since a single framesvg is shared between all tasks, we could have to resize it even if there wasn't a resizeevent
+    if (size().toSize() != itemBackground->frameSize()) {
+        resizeBackground(size().toSize());
+    }
+
     QPixmap oldBackground;
 
     if (option->state & QStyle::State_Sunken) {
@@ -618,6 +623,11 @@ void AbstractTaskItem::drawBackground(QPainter *painter, const QStyleOptionGraph
     }
 
     itemBackground->setElementPrefix(m_backgroundPrefix);
+    //since a single framesvg is shared between all tasks, we could have to resize it even if there wasn't a resizeevent
+    if (size().toSize() != itemBackground->frameSize()) {
+        resizeBackground(size().toSize());
+    }
+
     QPixmap result = Plasma::PaintUtils::transition(oldBackground, itemBackground->framePixmap(), m_alpha);
 
     if (result.size() == m_activeRect.size().toSize()) {
@@ -625,7 +635,6 @@ void AbstractTaskItem::drawBackground(QPainter *painter, const QStyleOptionGraph
     } else {
         painter->drawPixmap(QPoint(0,0), result);
     }
-
 }
 
 void AbstractTaskItem::drawTask(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *)
@@ -649,7 +658,10 @@ void AbstractTaskItem::drawTask(QPainter *painter, const QStyleOptionGraphicsIte
         const bool fadingBg = m_backgroundFadeAnim && m_backgroundFadeAnim->state() == QAbstractAnimation::Running;
         if ((!fadingBg && !(option->state & QStyle::State_MouseOver)) ||
             (m_oldBackgroundPrefix != "hover" && m_backgroundPrefix != "hover")) {
-            icon().paint(painter, iconR.toRect());
+            // QIcon::paint does some alignment work and can lead to funny
+            // things when icon().size() != iconR.toRect().size()
+            QPixmap result = icon().pixmap(iconR.toRect().size());
+            painter->drawPixmap(iconR.topLeft(), result);
         } else {
             KIconEffect *effect = KIconLoader::global()->iconEffect();
             QPixmap result = icon().pixmap(iconR.toRect().size());
@@ -974,7 +986,7 @@ void AbstractTaskItem::setGeometry(const QRectF& geometry)
     }
 }
 
-QRectF AbstractTaskItem::iconRect(const QRectF &b) const
+QRectF AbstractTaskItem::iconRect(const QRectF &b)
 {
     QRectF bounds(b);
     const int right = bounds.right();
@@ -1001,12 +1013,16 @@ QRectF AbstractTaskItem::iconRect(const QRectF &b) const
         }
     }
 
+    if (iconSize != m_lastIconSize) {
+        m_cachedShadow = QPixmap();
+    }
+    m_lastIconSize = iconSize;
     return QStyle::alignedRect(QApplication::layoutDirection(),
                                (m_showText ? Qt::AlignLeft : Qt::AlignCenter) | Qt::AlignVCenter,
                                iconSize, bounds.toRect());
 }
 
-QRectF AbstractTaskItem::expanderRect(const QRectF &bounds) const
+QRectF AbstractTaskItem::expanderRect(const QRectF &bounds)
 {
     const TaskGroupItem *groupItem = qobject_cast<const TaskGroupItem *>(this);
     if (!groupItem) {
@@ -1024,7 +1040,7 @@ QRectF AbstractTaskItem::expanderRect(const QRectF &bounds) const
                                expanderSize, bounds.toRect());
 }
 
-QRectF AbstractTaskItem::textRect(const QRectF &bounds) const
+QRectF AbstractTaskItem::textRect(const QRectF &bounds)
 {
     QSize size(bounds.size().toSize());
     QRectF effectiveBounds(bounds);
