@@ -1,5 +1,6 @@
 /*
  *   Copyright 2006 Aaron Seigo <aseigo@kde.org>
+ *   Copyright 2010 Chani Armitage <chani@kde.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -64,20 +65,25 @@
 #include <Plasma/AccessManager>
 #include <Plasma/AuthorizationManager>
 #include <Plasma/Containment>
+#include <Plasma/Context>
 #include <Plasma/Dialog>
 #include <Plasma/Theme>
 #include <Plasma/Wallpaper>
+#include <Plasma/WindowEffects>
 
 #include <kephal/screens.h>
 
 #include <plasmagenericshell/backgrounddialog.h>
+#include "kactivitycontroller.h"
 
+#include "activity.h"
 #include "appadaptor.h"
 #include "controllerwindow.h"
 #include "checkbox.h"
 #include "desktopcorona.h"
 #include "desktopview.h"
 #include "interactiveconsole.h"
+#include "kactivityinfo.h"
 #include "panelview.h"
 #include "plasma-shell-desktop.h"
 #include "toolbutton.h"
@@ -99,13 +105,13 @@ PlasmaApp* PlasmaApp::self()
 PlasmaApp::PlasmaApp()
     : KUniqueApplication(),
       m_corona(0),
-      m_controllerDialog(0),
-      m_zoomLevel(Plasma::DesktopZoom),
       m_panelHidden(0),
       m_mapper(new QSignalMapper(this)),
       m_startupSuspendWaitCount(0),
-      m_ignoreDashboardClosures(false)
+      m_ignoreDashboardClosures(false),
+      m_pendingFixedDashboard(false)
 {
+    kDebug() << "!!{} STARTUP TIME" << QTime().msecsTo(QTime::currentTime()) << "plasma app ctor start" << "(line:" << __LINE__ << ")";
     PlasmaApp::suspendStartup(true);
     KGlobal::locale()->insertCatalog("libplasma");
     KGlobal::locale()->insertCatalog("plasmagenericshell");
@@ -151,10 +157,11 @@ PlasmaApp::PlasmaApp()
     // - aseigo.
     KGlobalAccel::cleanComponent("plasma");
 
-    m_panelViewCreationTimer = new QTimer(this);
-    m_panelViewCreationTimer->setSingleShot(true);
-    m_panelViewCreationTimer->setInterval(0);
-    connect(m_panelViewCreationTimer, SIGNAL(timeout()), this, SLOT(createWaitingPanels()));
+    m_panelViewCreationTimer.setSingleShot(true);
+    m_panelViewCreationTimer.setInterval(0);
+
+    m_desktopViewCreationTimer.setSingleShot(true);
+    m_desktopViewCreationTimer.setInterval(0);
 
     new PlasmaAppAdaptor(this);
     QDBusConnection::sessionBus().registerObject("/App", this);
@@ -246,6 +253,7 @@ PlasmaApp::PlasmaApp()
         Plasma::AuthorizationManager::PinPairing);
 
     QTimer::singleShot(0, this, SLOT(setupDesktop()));
+    kDebug() << "!!{} STARTUP TIME" << QTime().msecsTo(QTime::currentTime()) << "plasma app ctor end" << "(line:" << __LINE__ << ")";
 }
 
 PlasmaApp::~PlasmaApp()
@@ -267,6 +275,8 @@ void PlasmaApp::setupDesktop()
     m_XdndVersionAtom = (Atom)xdndversion;
 #endif
 
+    m_primaryScreen = Kephal::ScreenUtils::primaryScreenId();
+
     // intialize the default theme and set the font
     Plasma::Theme *theme = Plasma::Theme::defaultTheme();
     theme->setFont(AppSettings::desktopFont());
@@ -277,6 +287,7 @@ void PlasmaApp::setupDesktop()
 
     Kephal::Screens *screens = Kephal::Screens::self();
     connect(screens, SIGNAL(screenRemoved(int)), SLOT(screenRemoved(int)));
+    connect(screens, SIGNAL(screenAdded(Kephal::Screen*)), SLOT(screenAdded(Kephal::Screen*)));
 
     if (AppSettings::perVirtualDesktopViews()) {
         connect(KWindowSystem::self(), SIGNAL(numberOfDesktopsChanged(int)),
@@ -290,6 +301,13 @@ void PlasmaApp::setupDesktop()
     desktop()->setPalette(palette);
 
     connect(this, SIGNAL(aboutToQuit()), this, SLOT(cleanup()));
+    kDebug() << "!!{} STARTUP TIME" << QTime().msecsTo(QTime::currentTime()) << "Plasma App SetupDesktop()" << "(line:" << __LINE__ << ")";
+
+    // now connect up the creation timers and start them to get the views created
+    connect(&m_panelViewCreationTimer, SIGNAL(timeout()), this, SLOT(createWaitingPanels()));
+    connect(&m_desktopViewCreationTimer, SIGNAL(timeout()), this, SLOT(createWaitingDesktops()));
+    m_panelViewCreationTimer.start();
+    m_desktopViewCreationTimer.start();
 }
 
 void PlasmaApp::quit()
@@ -438,17 +456,36 @@ void PlasmaApp::panelHidden(bool hidden)
     }
 }
 
-Plasma::ZoomLevel PlasmaApp::desktopZoomLevel() const
-{
-    return m_zoomLevel;
-}
-
 QList<PanelView*> PlasmaApp::panelViews() const
 {
     return m_panels;
 }
 
 void PlasmaApp::showWidgetExplorer(int screen, Plasma::Containment *containment)
+{
+    showController(screen, containment, true);
+}
+
+//FIXME it'd be easier if we knew which containment triggered this action
+void PlasmaApp::showActivityManager()
+{
+    //try to find the "active" containment
+    int currentScreen = Kephal::ScreenUtils::screenId(QCursor::pos());
+    int currentDesktop = -1;
+    if (AppSettings::perVirtualDesktopViews()) {
+        currentDesktop = KWindowSystem::currentDesktop()-1;
+    }
+    Plasma::Containment *containment=m_corona->containmentForScreen(currentScreen, currentDesktop);
+
+    showController(currentScreen, containment, false);
+}
+
+void PlasmaApp::showActivityManager(int screen, Plasma::Containment *containment)
+{
+    showController(screen, containment, false);
+}
+
+void PlasmaApp::showController(int screen, Plasma::Containment *containment, bool widgetExplorerMode)
 {
     if (!containment) {
         kDebug() << "no containment";
@@ -466,7 +503,11 @@ void PlasmaApp::showWidgetExplorer(int screen, Plasma::Containment *containment)
 
     controller->setContainment(containment);
     controller->setLocation(containment->location());
-    controller->showWidgetExplorer();
+    if (widgetExplorerMode) {
+        controller->showWidgetExplorer();
+    } else {
+        controller->showActivityManager();
+    }
     controller->resize(controller->sizeHint());
 
     bool moved = false;
@@ -483,17 +524,19 @@ void PlasmaApp::showWidgetExplorer(int screen, Plasma::Containment *containment)
 
     if (!moved) {
         // set it to the bottom of the screen as we have no better hints to go by
-        QRect geom = Kephal::ScreenUtils::screenGeometry(screen);
+        QRect geom = Kephal::Screens::self()->screen(screen)->geom();
         controller->resize(controller->sizeHint());
         controller->setGeometry(geom.x(), geom.bottom() - controller->height(), geom.width(), controller->height());
     }
 
     controller->show();
+    Plasma::WindowEffects::slideWindow(controller, Plasma::BottomEdge);
     KWindowSystem::setOnAllDesktops(controller->winId(), true);
     KWindowSystem::activateWindow(controller->winId());
+    KWindowSystem::setState(controller->winId(), NET::SkipTaskbar | NET::SkipPager | NET::Sticky | NET::KeepAbove);
 }
 
-void PlasmaApp::hideWidgetExplorer(int screen)
+void PlasmaApp::hideController(int screen)
 {
     QWeakPointer<ControllerWindow> controller = m_widgetExplorers.value(screen);
     if (controller) {
@@ -524,7 +567,7 @@ PanelView *PlasmaApp::findPanelForTrigger(WId trigger) const
 
 bool PlasmaApp::x11EventFilter(XEvent *event)
 {
-    if (m_panelHidden &&
+    if (m_panelHidden > 0 &&
         (event->type == ClientMessage ||
          (event->xany.send_event != True && (event->type == EnterNotify ||
                                              event->type == MotionNotify)))) {
@@ -559,6 +602,8 @@ bool PlasmaApp::x11EventFilter(XEvent *event)
                 const unsigned long *l = (const unsigned long *)event->xclient.data.l;
                 if (dndPosition) {
                     p = QPoint((l[2] & 0xffff0000) >> 16, l[2] & 0x0000ffff);
+                } else {
+                    p = QCursor::pos();
                 }
 
                 XClientMessageEvent response;
@@ -592,8 +637,6 @@ bool PlasmaApp::x11EventFilter(XEvent *event)
                 //kDebug() << "motion" << motion->x << motion->y << panel->location();
                 panel->hintOrUnhide(QPoint(motion->x_root, motion->y_root));
             }
-
-            return true;
         }
     }
 
@@ -603,37 +646,123 @@ bool PlasmaApp::x11EventFilter(XEvent *event)
 
 void PlasmaApp::screenRemoved(int id)
 {
-    kDebug() << id;
+    kDebug() << "@@@@" << id;
     QMutableListIterator<DesktopView *> it(m_desktops);
     while (it.hasNext()) {
         DesktopView *view = it.next();
         if (view->screen() == id) {
             // the screen was removed, so we'll destroy the
             // corresponding view
-            kDebug() << "removing the view for screen" << id;
+            kDebug() << "@@@@removing the view for screen" << id;
             view->setContainment(0);
             it.remove();
             delete view;
         }
     }
 
+    Kephal::Screen *primary = Kephal::Screens::self()->primaryScreen();
+    QList<Kephal::Screen *> screens = Kephal::Screens::self()->screens();
+    screens.removeAll(primary);
+
+    // Now we process panels: if there is room on another screen for the panel,
+    // we migrate the panel there, otherwise the view is deleted. The primary
+    // screen is preferred in all cases.
     QMutableListIterator<PanelView*> pIt(m_panels);
     while (pIt.hasNext()) {
         PanelView *panel = pIt.next();
         if (panel->screen() == id) {
-            kDebug() << "removing a panel for screen" << id;
-            panel->setContainment(0);
-            pIt.remove();
-            delete panel;
+            Kephal::Screen *moveTo = 0;
+            if (canRelocatePanel(panel, primary)) {
+                moveTo = primary;
+            } else {
+                foreach (Kephal::Screen *screen, screens) {
+                    if (canRelocatePanel(panel, screen)) {
+                        moveTo = screen;
+                        break;
+                    }
+                }
+            }
+
+            if (moveTo) {
+                panel->migrateTo(moveTo->id());
+            } else {
+                pIt.remove();
+                delete panel;
+                continue;
+            }
+        }
+
+        panel->updateStruts();
+    }
+}
+
+void PlasmaApp::screenAdded(Kephal::Screen *screen)
+{
+    foreach (Plasma::Containment *containment, corona()->containments()) {
+        if (isPanelContainment(containment) && containment->screen() == screen->id()) {
+            m_panelsWaiting << containment;
+            m_panelViewCreationTimer.start();
+        }
+    }
+
+    foreach (PanelView *view, m_panels) {
+        if (view->migratedFrom(screen->id())) {
+            view->migrateTo(screen->id());
         }
     }
 }
 
+bool PlasmaApp::canRelocatePanel(PanelView * view, Kephal::Screen *screen)
+{
+    if (!screen || !view->containment()) {
+        return false;
+    }
+
+    QRect newGeom = view->geometry();
+    switch (view->location()) {
+        case Plasma::TopEdge:
+            newGeom.setY(screen->geom().y());
+            newGeom.setX(view->offset());
+            break;
+        case Plasma::BottomEdge:
+            newGeom.setY(screen->geom().bottom() - newGeom.height());
+            newGeom.setX(view->offset());
+            break;
+        case Plasma::LeftEdge:
+            newGeom.setX(screen->geom().left());
+            newGeom.setY(view->offset());
+            break;
+        case Plasma::RightEdge:
+            newGeom.setX(screen->geom().right() - newGeom.width());
+            newGeom.setY(view->offset());
+            break;
+        default:
+            break;
+    }
+
+    kDebug() << "testing:" << screen->id() << view << view->geometry() << view->location() << newGeom;
+    foreach (PanelView *pv, m_panels) {
+        kDebug() << pv << pv->screen() << pv->screen() << pv->location() << pv->geometry();
+        if (pv != view &&
+            pv->screen() == screen->id() &&
+            pv->location() == view->location() &&
+            pv->geometry().intersects(newGeom)) {
+            return false;
+            break;
+        }
+    }
+
+    return true;
+}
+
+
 DesktopView* PlasmaApp::viewForScreen(int screen, int desktop) const
 {
     foreach (DesktopView *view, m_desktops) {
-        //kDebug() << "comparing" << view->screen() << screen;
-        if (view->screen() == screen && (desktop < 0 || view->desktop() == desktop)) {
+        if (view->containment()) {
+            kDebug() << "comparing" << view->containment()->screen() << screen;
+        }
+        if (view->containment() && view->containment()->screen() == screen && (desktop < 0 || view->containment()->desktop() == desktop)) {
             return view;
         }
     }
@@ -652,6 +781,8 @@ Plasma::Corona* PlasmaApp::corona()
         connect(c, SIGNAL(configSynced()), this, SLOT(syncConfig()));
         connect(c, SIGNAL(immutabilityChanged(Plasma::ImmutabilityType)),
                 this, SLOT(updateActions(Plasma::ImmutabilityType)));
+        connect(c, SIGNAL(screenOwnerChanged(int,int,Plasma::Containment*)),
+                this, SLOT(containmentScreenOwnerChanged(int,int,Plasma::Containment*)));
 
         foreach (DesktopView *view, m_desktops) {
             connect(c, SIGNAL(screenOwnerChanged(int,int,Plasma::Containment*)),
@@ -659,30 +790,22 @@ Plasma::Corona* PlasmaApp::corona()
         }
 
         //actions!
-        KAction *activityAction = c->addAction("add sibling containment");
-        connect(activityAction, SIGNAL(triggered()), this, SLOT(addContainment()));
-        activityAction->setText(i18n("Add Activity"));
-        activityAction->setIcon(KIcon("list-add"));
-        activityAction->setData(Plasma::AbstractToolBox::AddTool);
+        KAction *activityAction = c->addAction("manage activities");
+        connect(activityAction, SIGNAL(triggered()), this, SLOT(showActivityManager()));
+        activityAction->setText(i18n("Activities..."));
+        activityAction->setIcon(KIcon("preferences-activities"));
+        activityAction->setData(Plasma::AbstractToolBox::ConfigureTool);
         activityAction->setShortcut(KShortcut("alt+d, alt+a"));
         activityAction->setShortcutContext(Qt::ApplicationShortcut);
-
-        KAction *zoomAction = c->addAction("zoom out");
-        zoomAction->setText(i18n("Zoom Out"));
-        zoomAction->setIcon(KIcon("zoom-out"));
-        zoomAction->setData(Plasma::AbstractToolBox::ControlTool);
-        connect(zoomAction, SIGNAL(triggered()), this, SLOT(zoomOut()));
-        zoomAction->setShortcut(KShortcut("alt+d, -"));
+        activityAction->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_Q));
 
         c->updateShortcuts();
 
-        //add stuff to shortcut config
-        c->addShortcuts(DesktopView::shortcutActions(this));
-
         m_corona = c;
         c->setItemIndexMethod(QGraphicsScene::NoIndex);
         c->initializeLayout();
         c->processUpdateScripts();
+        c->checkActivities();
         c->checkScreens();
         foreach (Plasma::Containment *containment, c->containments()) {
             if (containment->screen() != -1 && containment->wallpaper()) {
@@ -752,6 +875,7 @@ bool PlasmaApp::isPanelContainment(Plasma::Containment *containment)
 
 void PlasmaApp::createView(Plasma::Containment *containment)
 {
+    kDebug() << "!!{} STARTUP TIME" << QTime().msecsTo(QTime::currentTime()) << "Plasma App createView() start" << "(line:" << __LINE__ << ")";
     kDebug() << "Containment name:" << containment->name()
              << "| type" << containment->containmentType()
              <<  "| screen:" << containment->screen()
@@ -764,8 +888,7 @@ void PlasmaApp::createView(Plasma::Containment *containment)
 
     if (isPanelContainment(containment)) {
         m_panelsWaiting << containment;
-        connect(containment, SIGNAL(destroyed(QObject*)), this, SLOT(waitingPanelRemoved(QObject*)));
-        m_panelViewCreationTimer->start();
+        m_panelViewCreationTimer.start();
     } else if (containment->screen() > -1 &&
                containment->screen() < Kephal::ScreenUtils::numScreens()) {
         if (AppSettings::perVirtualDesktopViews()) {
@@ -775,32 +898,8 @@ void PlasmaApp::createView(Plasma::Containment *containment)
             }
         }
 
-        KConfigGroup viewIds(KGlobal::config(), "ViewIds");
-        int id = viewIds.readEntry(QString::number(containment->id()), 0);
-        DesktopView *view = viewForScreen(containment->screen(),
-                                          AppSettings::perVirtualDesktopViews() ? containment->desktop() : -1);
-        if (view) {
-            kDebug() << "had a view for" << containment->screen() << containment->desktop();
-            // we already have a view for this screen
-            return;
-        }
-
-        kDebug() << "creating a new view for" << containment->screen() << containment->desktop()
-            << "and we have" << Kephal::ScreenUtils::numScreens() << "screens";
-
-        // we have a new screen. neat.
-        view = new DesktopView(containment, id, 0);
-        connect(view, SIGNAL(dashboardClosed()), this, SLOT(dashboardClosed()));
-        if (m_corona) {
-            connect(m_corona, SIGNAL(screenOwnerChanged(int,int,Plasma::Containment*)),
-                    view, SLOT(screenOwnerChanged(int,int,Plasma::Containment*)));
-            connect(m_corona, SIGNAL(shortcutsChanged()),
-                    view, SLOT(updateShortcuts()));
-        }
-
-        m_desktops.append(view);
-        view->show();
-        setWmClass(view->winId());
+        m_desktopsWaiting.append(containment);
+        m_desktopViewCreationTimer.start();
     }
 }
 
@@ -819,20 +918,142 @@ void PlasmaApp::setWmClass(WId id)
 
 void PlasmaApp::createWaitingPanels()
 {
-    foreach (Plasma::Containment *containment, m_panelsWaiting) {
-        disconnect(containment, SIGNAL(destroyed(QObject*)), this, SLOT(waitingPanelRemoved(QObject*)));
-        KConfigGroup viewIds(KGlobal::config(), "ViewIds");
-        int id = viewIds.readEntry(QString::number(containment->id()), 0);
-        if (containment->screen() < Kephal::ScreenUtils::numScreens()) {
-            PanelView *panelView = new PanelView(containment, id);
-            connect(panelView, SIGNAL(destroyed(QObject*)), this, SLOT(panelRemoved(QObject*)));
-            m_panels << panelView;
-            panelView->show();
-            setWmClass(panelView->winId());
+    if (m_panelsWaiting.isEmpty()) {
+        return;
+    }
+
+    const QList<QWeakPointer<Plasma::Containment> > containments = m_panelsWaiting;
+    m_panelsWaiting.clear();
+
+    foreach (QWeakPointer<Plasma::Containment> containmentPtr, containments) {
+        Plasma::Containment *containment = containmentPtr.data();
+        if (!containment) {
+            continue;
+        }
+
+        foreach (PanelView *view, m_panels) {
+            if (view->containment() == containment) {
+                continue;
+            }
+        }
+
+        if (containment->screen() < 0) {
+            continue;
+        }
+
+        // try to relocate the panel if it is on a now-non-existent screen
+        if (containment->screen() >= Kephal::ScreenUtils::numScreens()) {
+            m_panelRelocationCandidates << containment;
+            continue;
+        }
+
+        createPanelView(containment);
+    }
+
+    if (!m_panelRelocationCandidates.isEmpty()) {
+        QTimer::singleShot(0, this, SLOT(relocatePanels()));
+    }
+}
+
+void PlasmaApp::relocatePanels()
+{
+    // we go through relocatables last so that all other panels can be set up first,
+    // preventing panel creation ordering to trip up the canRelocatePanel algorithm
+    Kephal::Screen *primary = Kephal::Screens::self()->primaryScreen();
+    QList<Kephal::Screen *> screens = Kephal::Screens::self()->screens();
+    screens.removeAll(primary);
+
+    foreach (QWeakPointer<Plasma::Containment> c, m_panelRelocationCandidates) {
+        Plasma::Containment *containment = c.data();
+        if (!containment) {
+            continue;
+        }
+
+        Kephal::Screen *moveTo = 0;
+        PanelView *panelView = createPanelView(containment);
+        if (canRelocatePanel(panelView, primary)) {
+            moveTo = primary;
+        } else {
+            foreach (Kephal::Screen *screen, screens) {
+                if (canRelocatePanel(panelView, screen)) {
+                    moveTo = screen;
+                    break;
+                }
+            }
+        }
+
+        if (moveTo) {
+            panelView->migrateTo(moveTo->id());
+        } else {
+            m_panels.removeAll(panelView);
+            delete panelView;
         }
     }
 
-    m_panelsWaiting.clear();
+    m_panelRelocationCandidates.clear();
+}
+
+PanelView *PlasmaApp::createPanelView(Plasma::Containment *containment)
+{
+    KConfigGroup viewIds(KGlobal::config(), "ViewIds");
+    const int id = viewIds.readEntry(QString::number(containment->id()), 0);
+    PanelView *panelView = new PanelView(containment, id);
+
+    connect(panelView, SIGNAL(destroyed(QObject*)), this, SLOT(panelRemoved(QObject*)));
+    m_panels << panelView;
+    panelView->show();
+    setWmClass(panelView->winId());
+    return panelView;
+}
+
+void PlasmaApp::createWaitingDesktops()
+{
+    const QList<QWeakPointer<Plasma::Containment> > containments = m_desktopsWaiting;
+    m_desktopsWaiting.clear();
+
+    foreach (QWeakPointer<Plasma::Containment> weakContainment, containments) {
+        if (weakContainment) {
+            Plasma::Containment *containment = weakContainment.data();
+            KConfigGroup viewIds(KGlobal::config(), "ViewIds");
+            const int id = viewIds.readEntry(QString::number(containment->id()), 0);
+
+            const int desktop = AppSettings::perVirtualDesktopViews() ? containment->desktop() : -1;
+            if (desktop >= KWindowSystem::numberOfDesktops()) {
+                kDebug() << "not creating a view on desktop" << desktop << " as it does not exist";
+                continue;
+            }
+
+            const int screen = containment->screen();
+            if (screen >= Kephal::ScreenUtils::numScreens()) {
+                kDebug() << "not creating a view on screen" << screen << "as it does not exist";
+                continue;
+            }
+
+            DesktopView *view = viewForScreen(screen, desktop);
+
+            if (view) {
+                kDebug() << "already had a view for" << containment->screen() << containment->desktop();
+                // we already have a view for this screen
+                continue;
+            }
+
+            kDebug() << "creating a new view for" << containment->screen() << containment->desktop()
+                     << "and we have" << Kephal::ScreenUtils::numScreens() << "screens";
+
+            // we have a new screen. neat.
+            view = new DesktopView(containment, id, 0);
+            connect(view, SIGNAL(dashboardClosed()), this, SLOT(dashboardClosed()));
+            if (m_corona) {
+                connect(m_corona, SIGNAL(screenOwnerChanged(int,int,Plasma::Containment*)),
+                        view, SLOT(screenOwnerChanged(int,int,Plasma::Containment*)));
+            }
+
+            m_desktops.append(view);
+            view->show();
+            setWmClass(view->winId());
+        }
+    }
+    setFixedDashboard(fixedDashboard());
 }
 
 void PlasmaApp::containmentAdded(Plasma::Containment *containment)
@@ -847,16 +1068,22 @@ void PlasmaApp::containmentAdded(Plasma::Containment *containment)
     }
 
     createView(containment);
+
     disconnect(containment, 0, this, 0);
-    connect(containment, SIGNAL(zoomRequested(Plasma::Containment*,Plasma::ZoomDirection)),
-            this, SLOT(zoom(Plasma::Containment*,Plasma::ZoomDirection)));
     connect(containment, SIGNAL(configureRequested(Plasma::Containment*)),
             this, SLOT(configureContainment(Plasma::Containment*)));
 
-    if (containment->containmentType() == Plasma::Containment::DesktopContainment &&
-        m_zoomLevel == Plasma::DesktopZoom) {
-        foreach (QAction *action, m_corona->actions()) {
-            containment->addToolBoxAction(action);
+    if ((containment->containmentType() == Plasma::Containment::DesktopContainment ||
+         containment->containmentType() == Plasma::Containment::CustomContainment)) {
+        QAction *a = containment->action("remove");
+        if (a) {
+            delete a; //activities handle removal now
+        }
+
+        if (containment->containmentType() == Plasma::Containment::DesktopContainment) {
+            foreach (QAction *action, m_corona->actions()) {
+                containment->addToolBoxAction(action);
+            }
         }
     }
 
@@ -865,6 +1092,33 @@ void PlasmaApp::containmentAdded(Plasma::Containment *containment)
     }
 }
 
+void PlasmaApp::containmentScreenOwnerChanged(int wasScreen, int isScreen, Plasma::Containment *containment)
+{
+    Q_UNUSED(wasScreen)
+    kDebug() << "@@@was" << wasScreen << "is" << isScreen << (QObject*)containment;
+
+    if (isScreen < 0) {
+        kDebug() << "@@@screen<0";
+        return;
+    }
+    if (isPanelContainment(containment)) {
+        kDebug() << "@@@isPanel";
+        return;
+    }
+
+    bool pvd = AppSettings::perVirtualDesktopViews();
+    foreach (DesktopView *view, m_desktops) {
+        if (view->screen() == isScreen && (!pvd || view->desktop() == containment->desktop())) {
+            kDebug() << "@@@@found view" << view;
+            return;
+        }
+    }
+
+    kDebug() << "@@@@appending";
+    m_desktopsWaiting.append(containment);
+    m_desktopViewCreationTimer.start();
+}
+
 void PlasmaApp::configureContainment(Plasma::Containment *containment)
 {
     const QString id = "plasma_containment_settings_" + QString::number(containment->id());
@@ -893,6 +1147,10 @@ void PlasmaApp::configureContainment(Plasma::Containment *containment)
         configDialog = new BackgroundDialog(resolution, containment, view, 0, id, nullManager);
         configDialog->setAttribute(Qt::WA_DeleteOnClose);
 
+        Activity *activity = m_corona->activity(containment->context()->currentActivityId());
+        Q_ASSERT(activity);
+        connect(configDialog, SIGNAL(containmentPluginChanged(Plasma::Containment*)),
+                activity, SLOT(replaceContainment(Plasma::Containment*)));
         connect(configDialog, SIGNAL(destroyed(QObject*)), nullManager, SLOT(deleteLater()));
     }
 
@@ -901,170 +1159,27 @@ void PlasmaApp::configureContainment(Plasma::Containment *containment)
     KWindowSystem::activateWindow(configDialog->winId());
 }
 
-void PlasmaApp::addContainment()
+void PlasmaApp::cloneCurrentActivity()
 {
-    //try to find the "active" containment to get a plugin name
-    int currentScreen = Kephal::ScreenUtils::screenId(QCursor::pos());
-    int currentDesktop = -1;
-    if (AppSettings::perVirtualDesktopViews()) {
-        currentDesktop = KWindowSystem::currentDesktop()-1;
-    }
-    Plasma::Containment *fromContainment=m_corona->containmentForScreen(currentScreen, currentDesktop);
+    KActivityController controller;
+    //getting the current activity is *so* much easier than the current containment(s) :) :)
+    QString oldId = controller.currentActivity();
+    Activity *oldActivity = m_corona->activity(oldId);
+    QString newId = controller.addActivity(i18nc("%1 is the activity name", "copy of %1", oldActivity->name()));
 
-    QString plugin = fromContainment ? fromContainment->pluginName() : QString();
-    Plasma::Containment *c = m_corona->addContainment(plugin);
+    QString file = "activities/";
+    file += newId;
+    KConfig external(file, KConfig::SimpleConfig, "appdata");
 
-    if (c && fromContainment) {
-        foreach (DesktopView *view, m_desktops) {
-            if (view->containment() == fromContainment){
-                view->setContainment(c);
-                return;
-            }
-        }
+    //copy the old config to the new location
+    oldActivity->save(external);
+    //kDebug() << "saved to" << file;
 
-        // if we reach here, the containment isn't going to be taken over by the view,
-        // so we're going to resize it ourselves!
-        c->resize(fromContainment->size());
-    }
-}
-
-void PlasmaApp::zoomOut()
-{
-    zoom(0, Plasma::ZoomOut);
-}
-
-void PlasmaApp::zoom(Plasma::Containment *containment, Plasma::ZoomDirection direction)
-{
-    if (direction == Plasma::ZoomIn) {
-        zoomIn(containment);
-        foreach (DesktopView *view, m_desktops) {
-            view->zoomIn(m_zoomLevel);
-        }
-
-        if (m_zoomLevel == Plasma::DesktopZoom) {
-            int currentDesktop = -1;
-            if (AppSettings::perVirtualDesktopViews()) {
-                currentDesktop = KWindowSystem::currentDesktop()-1;
-            }
-
-            DesktopView *view = viewForScreen(desktop()->screenNumber(QCursor::pos()), currentDesktop);
-
-            if (view && view->containment() != containment) {
-                // zooming in all the way, so lets swap containments about if need be
-                view->setContainment(containment);
-            }
-            //FIXME: toolbox must be fixed in the proper way!
-            foreach (Plasma::Containment *cont, m_corona->containments()) {
-                cont->openToolBox();
-                cont->closeToolBox();
-            }
-        }
-    } else if (direction == Plasma::ZoomOut) {
-        zoomOut(containment);
-        foreach (DesktopView *view, m_desktops) {
-            view->zoomOut(m_zoomLevel);
-        }
-    }
-}
-
-void PlasmaApp::zoomIn(Plasma::Containment *containment)
-{
-    bool isMutable = m_corona->immutability() == Plasma::Mutable;
-    bool zoomIn = true;
-    bool zoomOut = true;
-    bool addSibling = isMutable;
-    bool remove = isMutable;
-
-    if (m_zoomLevel == Plasma::GroupZoom) {
-        setControllerVisible(false);
-        m_zoomLevel = Plasma::DesktopZoom;
-        containment->closeToolBox();
-        zoomIn = false;
-        //remove = false;
-    } else if (m_zoomLevel == Plasma::OverviewZoom) {
-        m_zoomLevel = Plasma::GroupZoom;
-    }
-
-    //make sure everybody can zoom out again
-    foreach (Plasma::Containment *c, m_corona->containments()) {
-        if (isPanelContainment(c)) {
-            continue;
-        }
-
-        if (m_zoomLevel == Plasma::DesktopZoom) {
-            foreach (QAction *action, m_corona->actions()) {
-                c->addToolBoxAction(action);
-            }
-        }
-
-        c->enableAction("zoom in", zoomIn);
-        c->enableAction("remove", remove);
-        c->enableAction("add widgets", isMutable);
-    }
-    m_corona->enableAction("zoom out", zoomOut);
-    m_corona->enableAction("add sibling containment", addSibling);
-}
-
-void PlasmaApp::zoomOut(Plasma::Containment *)
-{
-    bool isMutable = m_corona->immutability() == Plasma::Mutable;
-    bool zoomIn = true;
-    bool zoomOut = true;
-    bool addSibling = isMutable;
-    bool addWidgets = isMutable;
-
-    if (m_zoomLevel == Plasma::DesktopZoom) {
-        setControllerVisible(true);
-        m_zoomLevel = Plasma::GroupZoom;
-    } else if (m_zoomLevel == Plasma::GroupZoom) {
-        m_zoomLevel = Plasma::OverviewZoom;
-        zoomOut = false;
-        addWidgets = false;
-    }
-
-    //make sure everybody can zoom out again
-    foreach (Plasma::Containment *c, m_corona->containments()) {
-        if (isPanelContainment(c)) {
-            continue;
-        }
-
-        if (m_zoomLevel == Plasma::GroupZoom) {
-            foreach (QAction *action, m_corona->actions()) {
-                c->removeToolBoxAction(action);
-            }
-        }
-
-        c->enableAction("zoom in", zoomIn);
-        c->enableAction("remove", isMutable);
-        c->enableAction("add widgets", addWidgets);
-    }
-
-    m_corona->enableAction("zoom out", zoomOut);
-    m_corona->enableAction("add sibling containment", addSibling);
-}
-
-void PlasmaApp::setControllerVisible(bool show)
-{
-    if (show) {
-        if (!m_controllerDialog) {
-            m_controllerDialog = new Plasma::Dialog;
-            QVBoxLayout *layout = new QVBoxLayout(m_controllerDialog);
-
-            foreach (QAction *action, m_corona->actions()) {
-                ToolButton *actionButton = new ToolButton(m_controllerDialog);
-                actionButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
-                actionButton->setDefaultAction(action);
-                layout->addWidget(actionButton);
-            }
-        }
-
-        m_controllerDialog->show();
-    } else if (!show) {
-        delete m_controllerDialog;
-        m_controllerDialog = 0;
-    }
+    //load the new one
+    controller.setCurrentActivity(newId);
 }
 
+//TODO accomodate activities
 void PlasmaApp::setPerVirtualDesktopViews(bool perDesktopViews)
 {
     AppSettings::setPerVirtualDesktopViews(perDesktopViews);
@@ -1073,10 +1188,13 @@ void PlasmaApp::setPerVirtualDesktopViews(bool perDesktopViews)
     disconnect(KWindowSystem::self(), SIGNAL(numberOfDesktopsChanged(int)),
                this, SLOT(checkVirtualDesktopViews(int)));
 
+    m_pendingFixedDashboard = fixedDashboard();
+
     if (perDesktopViews) {
         connect(KWindowSystem::self(), SIGNAL(numberOfDesktopsChanged(int)),
                 this, SLOT(checkVirtualDesktopViews(int)));
         checkVirtualDesktopViews(KWindowSystem::numberOfDesktops());
+        setFixedDashboard(m_pendingFixedDashboard);
     } else {
         QList<DesktopView *> perScreenViews;
         foreach (DesktopView *view, m_desktops) {
@@ -1090,14 +1208,6 @@ void PlasmaApp::setPerVirtualDesktopViews(bool perDesktopViews)
         m_desktops.clear();
         m_corona->checkScreens(true);
     }
-
-    foreach (DesktopView *view, m_desktops) {
-        view->zoomOut(m_zoomLevel);
-    }
-
-    foreach (Plasma::Containment *c, m_corona->containments()) {
-        c->enableAction("zoom in", true);
-    }
 }
 
 bool PlasmaApp::perVirtualDesktopViews() const
@@ -1124,10 +1234,22 @@ void PlasmaApp::checkVirtualDesktopViews(int numDesktops)
 
 void PlasmaApp::setFixedDashboard(bool fixedDashboard)
 {
+    //TODO: should probably have one dashboard containment per screen
     Plasma::Containment *c = 0;
+    m_pendingFixedDashboard = fixedDashboard;
     if (fixedDashboard) {
-        //avoid the containmentAdded signal being emitted
-        c = m_corona->addContainment("desktop");
+        foreach (Plasma::Containment *possibility, m_corona->containments()) {
+            if (possibility->pluginName() == "desktopDashboard") {
+                c = possibility;
+                break;
+            }
+        }
+
+        if (!c) {
+            //avoid the containmentAdded signal being emitted
+            c = m_corona->addContainment("desktopDashboard");
+        }
+
         m_corona->addOffscreenWidget(c);
     }
 
@@ -1138,21 +1260,27 @@ void PlasmaApp::setFixedDashboard(bool fixedDashboard)
             maxViewSize = view->size();
         }
     }
+
     if (fixedDashboard) {
         c->resize(maxViewSize);
     }
+
+    m_corona->requestConfigSync();
 }
 
 bool PlasmaApp::fixedDashboard() const
 {
-    bool dashboardFollowsDesktop = true;
+    if (m_desktops.isEmpty()) {
+        return m_pendingFixedDashboard;
+    }
+
     foreach (DesktopView *view, m_desktops) {
-        if (view->dashboardContainment()) {
-            dashboardFollowsDesktop = false;
-            break;
+        if (!view->dashboardFollowsDesktop()) {
+            return true;
         }
     }
-    return !dashboardFollowsDesktop;
+
+    return false;
 }
 
 void PlasmaApp::panelRemoved(QObject *panel)
@@ -1160,15 +1288,8 @@ void PlasmaApp::panelRemoved(QObject *panel)
     m_panels.removeAll((PanelView *)panel);
 }
 
-void PlasmaApp::waitingPanelRemoved(QObject *panelContainment)
-{
-    m_panelsWaiting.removeAll((Plasma::Containment *)panelContainment);
-}
-
 void PlasmaApp::updateActions(Plasma::ImmutabilityType immutability)
 {
-    bool enable = immutability == Plasma::Mutable;
-    m_corona->enableAction("add sibling containment", enable);
 }
 
 void PlasmaApp::remotePlasmoidAdded(Plasma::PackageMetadata metadata)
@@ -1208,5 +1329,18 @@ void PlasmaApp::plasmoidAccessFinished(Plasma::AccessAppletJob *job)
     }
 }
 
+void PlasmaApp::createActivity(const QString &plugin)
+{
+    KActivityController controller;
+    QString id = controller.addActivity(i18n("unnamed"));
+
+    Activity *a = m_corona->activity(id);
+    if (!a) {
+        kDebug() << "!*!*!*!*!*!*!*!*!*!**!*!*!*!!*!*!*!*!*!*";
+    }
+    a->setDefaultPlugin(plugin);
+
+    controller.setCurrentActivity(id);
+}
 
 #include "plasmaapp.moc"
