@@ -26,6 +26,8 @@
 
 #include <kdemacros.h>
 #include <KAboutData>
+#include <KAction>
+#include <KActionCollection>
 #include <KPluginFactory>
 #include <KNotification>
 #include <KIcon>
@@ -90,8 +92,12 @@ class PowerDevilDaemon::Private
 {
 public:
     explicit Private()
-            : notifier(Solid::Control::PowerManager::notifier())
+            : notifier(0)
+            , screenSaverIface(0)
+            , ksmServerIface(0)
             , currentConfig(0)
+            , lockHandler(0)
+            , notificationTimer(0)
             , status(PowerDevilDaemon::NoAction)
             , ckSessionInterface(0) {}
 
@@ -142,6 +148,18 @@ PowerDevilDaemon::PowerDevilDaemon(QObject *parent, const QList<QVariant>&)
 
     d->applicationData = KComponentData(aboutData);
 
+    QTimer::singleShot(0, this, SLOT(init()));
+}
+
+PowerDevilDaemon::~PowerDevilDaemon()
+{
+    delete d->ckSessionInterface;
+    delete d;
+}
+
+void PowerDevilDaemon::init()
+{
+    d->notifier = Solid::Control::PowerManager::notifier();
     d->lockHandler = new SuspensionLockHandler(this);
     d->notificationTimer = new QTimer(this);
 
@@ -180,6 +198,7 @@ PowerDevilDaemon::PowerDevilDaemon(QObject *parent, const QList<QVariant>&)
      *   d->kscreenSaverIface = new OrgKdeScreensaverInterface("org.freedesktop.ScreenSaver", "/ScreenSaver",
      *         QDBusConnection::sessionBus(), this);
     */
+    connect(d->notifier, SIGNAL(brightnessChanged(float)), SLOT(brightnessChangedSlot(float)));
     connect(d->notifier, SIGNAL(buttonPressed(int)), this, SLOT(buttonPressed(int)));
     connect(d->notifier, SIGNAL(batteryRemainingTimeChanged(int)),
             this, SLOT(batteryRemainingTimeChanged(int)));
@@ -198,12 +217,18 @@ PowerDevilDaemon::PowerDevilDaemon(QObject *parent, const QList<QVariant>&)
     QDBusConnection::sessionBus().registerService("org.kde.powerdevil");
     // All systems up Houston, let's go!
     refreshStatus();
-}
 
-PowerDevilDaemon::~PowerDevilDaemon()
-{
-    delete d->ckSessionInterface;
-    delete d;
+    KActionCollection* actionCollection = new KActionCollection( this );
+
+    KAction* globalAction = actionCollection->addAction("Increase Screen Brightness");
+    globalAction->setText(i18nc("Global shortcut", "Increase Screen Brightness"));
+    globalAction->setGlobalShortcut(KShortcut(Qt::Key_MonBrightnessUp), KAction::ShortcutTypes(KAction::ActiveShortcut | KAction::DefaultShortcut), KAction::NoAutoloading);
+    connect(globalAction, SIGNAL(triggered(bool)), SLOT(increaseBrightness()));
+
+    globalAction = actionCollection->addAction("Decrease Screen Brightness");
+    globalAction->setText(i18nc("Global shortcut", "Decrease Screen Brightness"));
+    globalAction->setGlobalShortcut(KShortcut(Qt::Key_MonBrightnessDown), KAction::ShortcutTypes(KAction::ActiveShortcut | KAction::DefaultShortcut), KAction::NoAutoloading);
+    connect(globalAction, SIGNAL(triggered(bool)), SLOT(decreaseBrightness()));
 }
 
 void PowerDevilDaemon::batteryRemainingTimeChanged(int time)
@@ -362,30 +387,8 @@ void PowerDevilDaemon::applyProfile()
 
     Solid::Control::PowerManager::setBrightness(settings->readEntry("brightness").toInt());
     d->brightness = settings->readEntry("brightness").toInt();
-    Solid::Control::PowerManager::setCpuFreqPolicy((Solid::Control::PowerManager::CpuFreqPolicy)
-            settings->readEntry("cpuPolicy").toInt());
 
-    QVariant var = settings->readEntry("disabledCPUs", QVariant());
-    QList<QVariant> list = var.toList();
-
-    foreach(const Solid::Device &device, Solid::Device::listFromType(Solid::DeviceInterface::Processor,
-                                                                     QString())) {
-        Solid::Device d = device;
-        Solid::Processor * processor = qobject_cast<Solid::Processor * > (d.asDeviceInterface(
-                Solid::DeviceInterface::Processor));
-
-        bool enable = true;
-
-        foreach(const QVariant &ent, list) {
-            if (processor->number() == ent.toInt()) {
-                enable = false;
-            }
-        }
-
-        Solid::Control::PowerManager::setCpuEnabled(processor->number(), enable);
-    }
-
-    Solid::Control::PowerManager::setScheme(settings->readEntry("scheme"));
+    Solid::Control::PowerManager::setPowerSave(settings->readEntry("setPowerSave", false));
 
     // Compositing!!
 
@@ -669,8 +672,8 @@ void PowerDevilDaemon::decreaseBrightness()
         return;
     }
 
-    int currentBrightness = qMax(0, (int)(Solid::Control::PowerManager::brightness() - 10));
-    Solid::Control::PowerManager::setBrightness(currentBrightness);
+    Solid::Control::PowerManager::brightnessKeyPressed(Solid::Control::PowerManager::Decrease);
+    emit brightnessChanged(qRound(Solid::Control::PowerManager::brightness()), true);
 }
 
 void PowerDevilDaemon::increaseBrightness()
@@ -679,8 +682,8 @@ void PowerDevilDaemon::increaseBrightness()
         return;
     }
 
-    int currentBrightness = qMin(100, (int)(Solid::Control::PowerManager::brightness() + 10));
-    Solid::Control::PowerManager::setBrightness(currentBrightness);
+    Solid::Control::PowerManager::brightnessKeyPressed(Solid::Control::PowerManager::Increase);
+    emit brightnessChanged(qRound(Solid::Control::PowerManager::brightness()), true);
 }
 
 void PowerDevilDaemon::shutdownNotification(bool automated)
@@ -970,7 +973,6 @@ void PowerDevilDaemon::poll(int id, int idle)
     } else {
         d->status = NoAction;
         KIdleTime::instance()->stopCatchingResumeEvent();
-        Solid::Control::PowerManager::setBrightness(settings->readEntry("brightness").toInt());
     }
 }
 
@@ -1168,36 +1170,6 @@ void PowerDevilDaemon::streamData()
     emit stateChanged(d->batteryPercent, d->isPlugged);
 }
 
-QVariantMap PowerDevilDaemon::getSupportedGovernors()
-{
-    QVariantMap retlist;
-
-    Solid::Control::PowerManager::CpuFreqPolicies policies =
-            Solid::Control::PowerManager::supportedCpuFreqPolicies();
-
-    if (policies & Solid::Control::PowerManager::Performance) {
-        retlist[i18n("Performance")] = (int) Solid::Control::PowerManager::Performance;
-    }
-
-    if (policies & Solid::Control::PowerManager::OnDemand) {
-        retlist[i18n("Dynamic (ondemand)")] = (int) Solid::Control::PowerManager::OnDemand;
-    }
-
-    if (policies & Solid::Control::PowerManager::Conservative) {
-        retlist[i18n("Dynamic (conservative)")] = (int) Solid::Control::PowerManager::Conservative;
-    }
-
-    if (policies & Solid::Control::PowerManager::Powersave) {
-        retlist[i18n("Powersave")] = (int) Solid::Control::PowerManager::Powersave;
-    }
-
-    if (policies & Solid::Control::PowerManager::Userspace) {
-        retlist[i18n("Userspace")] = (int) Solid::Control::PowerManager::Userspace;
-    }
-
-    return retlist;
-}
-
 QVariantMap PowerDevilDaemon::getSupportedSuspendMethods()
 {
     QVariantMap retlist;
@@ -1220,27 +1192,13 @@ QVariantMap PowerDevilDaemon::getSupportedSuspendMethods()
     return retlist;
 }
 
-QStringList PowerDevilDaemon::getSupportedSchemes()
-{
-    return Solid::Control::PowerManager::supportedSchemes();
-}
-
-void PowerDevilDaemon::setPowersavingScheme(const QString &scheme)
+void PowerDevilDaemon::setPowerSave(bool powersave)
 {
     if (!checkIfCurrentSessionActive()) {
         return;
     }
 
-    Solid::Control::PowerManager::setScheme(scheme);
-}
-
-void PowerDevilDaemon::setGovernor(int governor)
-{
-    if (!checkIfCurrentSessionActive()) {
-        return;
-    }
-
-    Solid::Control::PowerManager::setCpuFreqPolicy((Solid::Control::PowerManager::CpuFreqPolicy) governor);
+    Solid::Control::PowerManager::setPowerSave(powersave);
 }
 
 void PowerDevilDaemon::suspend(int method)
@@ -1461,4 +1419,8 @@ void PowerDevilDaemon::setUpConsoleKit()
                                          SLOT(refreshStatus()));
 }
 
+void PowerDevilDaemon::brightnessChangedSlot(float brightness) {
+    emit brightnessChanged(qRound(brightness), false);
+}
+
 #include "PowerDevilDaemon.moc"
