@@ -29,7 +29,6 @@
 #include <kcomponentdata.h>
 #include <kcmdlineargs.h>
 #include <klocale.h>
-#include <k3socketaddress.h>
 #include <kdebug.h>
 
 #include <solid/device.h>
@@ -612,7 +611,7 @@ bool SolidBluetooth::bluetoothInputSetup(const QString &deviceUbi)
 
     if (job==0)
     {
-        cerr << i18n("Error: unsupported operation!") << endl;
+        cerr << i18n("Error: unsupported operation.") << endl;
         return false;
     }
 
